
#ifndef __CAN_DATA_TYPES_H__
#define __CAN_DATA_TYPES_H__ 


const int RESET_NODE = 129;
const int RESET_COMMUNICATION = 130;
const int ENTER_PREOPERATIONAL = 128;
const int STOP_NODE = 2;
const int START_NODE = 1;

const int PARAMETER_NAME_LEN = 241;   // according to EDS specification: CiA Draft Standard Proposal 306

const int VAR    = 0x07;
const int ARRAY  = 0x08;
const int RECORD = 0x09;

const int CO_DATA_TYPE_UNKNOWN = -1; 
const int CO_NO_DATA = 0x00;  // used, when an entry is a header of the array
const int CO_BOOL1 = 0x01;
const int CO_INT8  = 0x02;
const int CO_INT16 = 0x03;
const int CO_INT32 = 0x04;
const int CO_UINT8 = 0x05;
const int CO_UINT16 = 0x06;
const int CO_UINT32 = 0x07;
const int CO_REAL32 = 0x08;
const int CO_VIS_STR = 0x09;
const int CO_OCTET_STR = 0x0A;
const int CO_UNICODE_STR = 0x0B;
const int CO_TIME_OF_DAY =  0x0C;
const int CO_TIME_DIFF = 0x0D;
const int CO_BIT_STR = 0x0E;
const int CO_DOMAIN =  0x0F;
const int CO_INT24 = 0x10;
const int CO_REAL64 = 0x11;
const int CO_INT40 = 0x12;
const int CO_INT48 = 0x13;
const int CO_INT56 = 0x14;
const int CO_INT64 = 0x15;
const int CO_UINT24 = 0x16;
const int CO_UINT40 = 0x18;
const int CO_UINT48 = 0x19;
const int CO_UINT56 = 0x1A;
const int CO_UINT64 = 0x1B;

typedef struct
{
  unsigned long ms;
  unsigned short days;
} TIMEINFO;

typedef union
{ 
  char                    valBOOL1;
  char                    valINT8;
  short                   valINT16;
  int                     valINT32;
  unsigned char           valUINT8;
  unsigned short          valUINT16;
  unsigned int            valUINT32;
  float                   valREAL32;
  char *                  valVIS_STR;     // must be handled by string allocation
  char *                  valOCTET_STR;   // must be handled by string allocation
  short *                 valUNICODE_STR; // must be handled by memory block allocation 
  TIMEINFO                valTIME_OF_DAY;
  TIMEINFO                valTIME_DIFF;
  char *                  valBIT_STR;
  void *                  valDOMAIN;      // must be handled by memory block allocation
  int                     valINT24;
  double                  valREAL64;
  long                    valINT40;
  long                    valINT48;
  long                    valINT56;
  long                    valINT64; 
  unsigned  int           valUINT24;
  unsigned long           valUINT40;
  unsigned long           valUINT48;
  unsigned long           valUINT56;
  unsigned long           valUINT64;
} OD_value;

#endif
