
#include "canMessageDispatcher.h"

CanMessageDispatcher::CanMessageDispatcher(CanDriver * canDriver)
{ 
  nCNIobjects = 0;
  canDrv = canDriver;
  memset( canMessageHandler, '\0', sizeof(canMessageHandler) );
  setName( "CanMessageDispatcher" );
}

bool CanMessageDispatcher::addMessageHandler( short cobId,  CanNodeImage * messageTarget, int (CanNodeImage::*messageHandler)(int data, CanMessage& cm), int data )
{
    if( cobId < 0 )    return false;
    if( cobId > 2047 ) return false;

    canMessageHandler[cobId].setData( data );
    canMessageHandler[cobId].setHandler( messageHandler );
    canMessageHandler[cobId].setTarget( messageTarget );
    return true;
}

void CanMessageDispatcher::activate( void )
{
 
  CanMessage  m;
  int data;
  int (CanNodeImage::*messageHandler)(int data, CanMessage& cm);
  CanNodeImage * target;
  if( canDrv->receive(m) ) 
    {    
      messageHandler = canMessageHandler[m.getId()].getHandler();
      target = canMessageHandler[m.getId()].getTarget();
      data = canMessageHandler[m.getId()].getData();
      if( (messageHandler != NULL) && (target != NULL)) (target->*messageHandler)(data,m);      
      else
      {
	if( Lg.getLoggingLevel() >= MSG_DBG )      
          {
            Lg << DBG << "unknown CANopen message" << endl;
            m.dump();
          }
      }
    }     
}

//
//void CanMessageDispatcher::activate( void )
//{
// 
//  CanMessage  m;
//  CanNodeImage * cni;
//
//  if( canDrv->receive(m) ) 
//    {    
//      switch( m.getId() )
//        {
//        case 0x00:   //from NMT master
//          break;
//
//        case 0x80:  // SYNC message
//          break;
//   
//        case 0x100: // TIME Stamp Message
//          break;
//
//        default:
//          unsigned short id = m.getId() & m.getIdMask();
//          if( ( cni = searchForCNI( id ) )!= NULL )
//            {             
//              switch(m.getId() - id )
//              {
//                case 0x80:
//                  if( m.getDataLen() == 0 ) cni->nmtHandler( m );  // some old devices sent it as a boot-up message
//                  else cni->emcyHandler( m );
//                  break;
//                case 0x180: 
//                  cni->pdoHandler( 0, m ); 
//                  break;
//                case 0x280:
//                  cni->pdoHandler( 1, m ); 
//                  break;
//                case 0x380:               
//                  cni->pdoHandler( 2, m ); 
//                  break;
//                case 0x480:
//                  cni->pdoHandler( 3, m );
//                  break;
//                case 0x580:
//                  cni->sdoHandler( m );
//                  break;
//                case 0x700:
//                  cni->nmtHandler( m );
//                  break;
//                default:
//                 if( Lg.getLoggingLevel() >= MSG_DBG )
//                  {
//                    Lg << DBG << "unknown CANopen message" << endl;
//                    m.dump();
//                  }
//                  break;                  
//              }
//            }
//          else
//            {
//              if( Lg.getLoggingLevel() >= MSG_DBG )
//                {
//                  Lg << DBG <<  "message from unknown object:" << endl;              
//                  m.dump(); 
//              }
//            }
//        }
//    }     
//}


bool CanMessageDispatcher::addCanNodeImage( CanNodeImage * cni )
{
  if( nCNIobjects == MAX_CNI_OBJECTS ) return false;
  cniList[nCNIobjects++] = cni;
  return true;
}

CanNodeImage * CanMessageDispatcher::searchForCNI(unsigned short canId )
{    
  for( int i = 0; i < nCNIobjects; i++ )
  {
    if( canId == cniList[i]->getNodeId() )
    {	
	return cniList[i];
    }
  }
  Lg << ERROR << "--Error: Dispatcher of " << canDrv->getDevice() << ", unknown Id = 0x" << hex << canId << dec << endl;
  return NULL;
}
