
/* 
   CANopen manager
   Karol, 24.11.2006
*/
#include <signal.h>
#include <unistd.h>

#include "configurator.h"
#include "coInit.h"

/*
      ---macros, constants, typedefs---      
*/

#define MAX_CAN_LINES 8    /*  8 is maximum number of CAN lines for the PCAN interfaces*/
#define MAX_DEVICE_NAME_LEN  128
#define DEFAULT_CAN_DEVICE  "/dev/pcan8"
/*
      ---non exported prototypes---   
*/
static void   terminate             ( int exit_code );
static void   print_version         ( void );
static void   set_signal_handlers   ( void );
static void   sig_exit              ( int sig );
static char * getsigname            ( int sig );
static void   sig_usr1              ( int sig );
static void   sig_usr2              ( int sig );

/*
      ---globals---
*/
typedef struct
        {
          int sigcode;
          char signame[10];
        } SIGNALS;
SIGNALS  Sigtab[] = 
              {  
               { SIGHUP,  "SIGHUP"  }, 
               { SIGINT,  "SIGINT"  },    
               { SIGTERM, "SIGTERM" },
               { SIGSEGV, "SIGSEGV" },
               { SIGBUS,  "SIGBUS"  },
               { SIGUSR1, "SIGUSR1" },
               { SIGUSR2, "SIGUSR2" },
               { 0,       ""        }
	      };

//TineFec TineFec;
TineServer * TineSrv[MAX_CAN_LINES];
Coman * CanOpenManager[MAX_CAN_LINES];
int BusCount = 0;

TimeSlice * ts1;
TimeSlice * ts2;
TimeSlice * ts3;

Log Lg( "TICOM" );

int coInit( Configurator * config )  // remove TineFec reference SWH 12.01.08
{                                       // but note that fecInit dependent on configurator   
  // or put these somewhere else? 
    print_version();
    set_signal_handlers();
    
    for(int i = 0; i < MAX_CAN_LINES; i++ ) CanOpenManager[i] = NULL;
    
    for(int i = 0; i < config->getCanBusCount(); i++)
    {
        TineSrv[i] = new TineServer(config->getTineServerName(i));
        // CanOpenManager[i] = new Coman(TineSrv[i], config->isEsdDriver() ? -1 : config->getCanInterfaceID(i));  
        CanOpenManager[i] = new Coman(TineSrv[i], config->getCanInterfaceID(i));  
        CanOpenManager[i]->driverSetup();
	CanOpenManager[i]->setBusName(config->getBusName(i));
	
        for(int j = 0; j  < config->getNodeCount(i); j++)
        {
            if(config->isCsvUsed()) CanOpenManager[i]->add(config->getNodeDCFname(i,j), config->getNodeName(i,j), config->getNodeId(i,j));		    
            else CanOpenManager[i]->add(config->getNodeDCFname(i,j));
        }
        for(int j = 0; j < CanOpenManager[i]->getNodesCount(); j++) TineSrv[i]->addDevice(CanOpenManager[i]->getNodeImage(j));        
    }
    
  for( int i = 0; i < config->getCanBusCount(); i++ )  CanOpenManager[i]->dump();
      
    
  // create 'time slices'
  ts1 = new TimeSlice();
  ts2 = new TimeSlice();
  ts3 = new TimeSlice();
   
  // instal Comans: assign each Coman devices to time slices 
  for(int i = 0; i < config->getCanBusCount(); i++)
  {
      ts1->add(CanOpenManager[i]->getCanDriver());
      ts2->add(CanOpenManager[i]->getCanMessageCollector());
      ts2->add(CanOpenManager[i]->getCanMessageDispatcher());
      ts2->add(CanOpenManager[i]->getSyncProducer());
      ts2->add(CanOpenManager[i]->getTimeProducer());
      for(int j = 0; j < CanOpenManager[i]->getNodesCount(); j++) ts3->add(CanOpenManager[i]->getNodeImage(j));
  }
    
  return BusCount = config->getCanBusCount();
}

Coman* getComan( int idx ) 
{
    return CanOpenManager[idx];
}

int isInitDone()
{
    if( BusCount == 0 ) return 0;
    for( int i = 0; i < BusCount; i++ )
    {
	if( CanOpenManager[i]->getUninitializedDeviceCount() > 0 ) return 0;
    }
    return 1;
}
// void fecInit(int initMode)
// { 
//     set_signal_handlers();
//     print_version();
//     TineFec.init(initMode);  // calls tine SysInit
// }

// void fecInit( int portOffset, char * fecName, int initMode )
// {
//     set_signal_handlers();
//     print_version();
//     TineFec.setPortOffset(portOffset);
//     TineFec.setFecName(fecName);
//     TineFec.init(initMode);  // calls tine SysInit
// }

void coActivate()
{
  ts1->activate();
  ts2->activate();
  ts3->activate();
}

//
//  parsing CSV file



void set_signal_handlers( void )
{
  signal( SIGHUP,  sig_exit ); 
  signal( SIGINT,  sig_exit );   
  signal( SIGTERM, sig_exit );  
  signal( SIGSEGV, sig_exit );  
  signal( SIGBUS,  sig_exit );  
  signal( SIGUSR1, sig_usr1 );
  signal( SIGUSR2, sig_usr2 );  
}

void print_version( void )
{
  Lg << INFO << "--- CANopen Manager vers." << __DATE__ << "," << __TIME__ <<" started." << endl;
} 

void terminate( int exit_code )
{
  Lg << INFO << "--- terminated. Exit code = " << exit_code  << endl;
  exit( exit_code );
}

void sig_usr1( int sig )
{
    int l = Lg.getLoggingLevel();
    if( l < 1 ) return;
    Lg.setLoggingLevel(--l);
    Lg << FATAL << "Logging level set to:" << l << endl;
}

void sig_usr2( int sig )
{
    int l = Lg.getLoggingLevel();
    if( l > 3 ) return;
    Lg.setLoggingLevel(++l);
    Lg << FATAL << "Logging level set to:" << l << endl;
}


void sig_exit( int sig )
{
  Lg << FATAL << "---Fatal: signal " <<  getsigname( sig ) << ",(" << sig << ") arrived."<< endl;
  terminate( sig );
}

char * getsigname( int sig )
{
     int i = 0;

     for( i = 0; Sigtab[i].sigcode; i++ )
          if( Sigtab[i].sigcode == sig ) return Sigtab[i].signame;

     return "***unknown***" ;
}
