
#include "shmlogger/shmlogger.h"

 
#include "coMan.h"


// to a proper linker behavior:
//TineServer * Coman::server = NULL;
//BUS_INFO Coman::busInfo;   


Coman::Coman(TineServer * srv) 
{
  construct(srv,PCAN_USB);
}

Coman::Coman(TineServer * srv, int canDevNo)
{
  construct(srv, canDevNo);
}

void Coman::construct(TineServer * srv, int canDevNo)
{
  setVersion();
  setName( "COMAN" );
  setFamilyName( "COMAN" );
  nodeImagesCount = 0;
  server = srv;
  if( canDevNo > -1 ) canDrv = new CanDriver(canDevNo, baudRate = 250000);  
  else canDrv = new CanDriverEsd(baudRate = 250000, canDevNo );
  cmc = new CanMessageCollector(canDrv);
  cmd = new CanMessageDispatcher(canDrv); 
  syncProducer = new SyncProducer(cmc);
  timeProducer = new TimeProducer(cmc);

  syncProducer->setInterval(3000);
  syncProducer->enable();

  timeProducer->setInterval(10000);
  timeProducer->enable();

  RegisterUserFunction( "dump", consoleRequest );

  buildProperties();
  
 // srv->init(this);               // set the eqm handler
  srv->addDevice(this);          // add Coman as a TINE device
  srv->addDevice(syncProducer);  // add syncProducer as a TINE device
  srv->addDevice(timeProducer);  // add timeProducer as a TINE device 
 
}

void Coman::buildProperties()
{
  fillstruct("BUSINFO",OFFSETIN(BUS_INFO,nNodes),1,CF_LONG);
  fillstruct("BUSINFO",OFFSETIN(BUS_INFO,nodeName),MAX_NODE_IMAGE_COUNT*33, CF_BYTE);
  fillstruct("BUSINFO",OFFSETIN(BUS_INFO,busName),33, CF_BYTE);
  fillstruct("BUSINFO",OFFSETIN(BUS_INFO,nodeState),MAX_NODE_IMAGE_COUNT,CF_LONG);
  fillstruct("BUSINFO",OFFSETIN(BUS_INFO,syncState),1,CF_LONG);
  fillstruct("BUSINFO",OFFSETIN(BUS_INFO,timeProducerState),1,CF_LONG);
  fillstruct("BUSINFO",OFFSETIN(BUS_INFO,busSpeed),1,CF_LONG);
  fillstruct("BUSINFO",OFFSETIN(BUS_INFO,ticomVersion),64,CF_BYTE); 
  fillstruct("BUSINFO",OFFSETIN(BUS_INFO,ticomStartTime),64,CF_BYTE);
  fillstruct("BUSINFO",sizeof(BUS_INFO),1,CF_NULL);

  DTYPE out, in;
  
  out.dArrayLength = 1;  out.dFormat = CF_STRUCT; strcpy( out.dTag, "BUSINFO" );
  in.dArrayLength = 0;  in.dFormat = CF_NULL; 
  addProperty( "BUSINFO", &out, &in, CA_READ, "CAN BUS DATA", &Coman::busInfoHandler );
  
  out.dArrayLength = 1;  out.dFormat = CF_LONG; 
  in.dArrayLength = 1;  in.dFormat = CF_LONG;
  addProperty( "ALLRESET", &out, &in, CA_WRITE, "RESET ALL CAN NODES",  &Coman::resetAllNodesHandler );  
  addProperty( "ALLPREOP", &out, &in, CA_WRITE, "SET ALL CAN NODES TO PREOPERATIONAL STATE",  &Coman::preoperationalAllNodesHandler );
  addProperty( "ALLOP", &out, &in, CA_WRITE, "SET ALL CAN NODES TO OPERATIONAL STATE",  &Coman::operationalAllNodesHandler ); 
  addProperty( "ALLSTOP", &out, &in, CA_WRITE, "SET ALL CAN NODES TO STOPPED STATE",  &Coman::stopAllNodesHandler ); 
  
  out.dArrayLength = 33;  out.dFormat = CF_TEXT; 
  in.dArrayLength = 0;  in.dFormat = CF_NULL;
  addProperty( "BUSNAME", &out, &in, CA_WRITE, "NAME OF THE BUS",  &Coman::getBusNameHandler ); 

}

CanNodeImage* Coman::add(char * dcfName, char * nodeName, int nodeId)
{
    CanNodeImage *cni = new CanNodeImage(cmd, cmc,dcfName, nodeName, nodeId );
    return add(cni);     
}

CanNodeImage* Coman::add(char * dcfName)
{
    CanNodeImage *cni = new CanNodeImage(cmd, cmc,dcfName);
    return add(cni);
}

CanNodeImage* Coman::add(CanNodeImage * cni)
{
  nodeImages[nodeImagesCount++] = cni;
  cmd->addCanNodeImage(cni);   // add a CAN node for message dispatching

  busInfo.nNodes = nodeImagesCount;
  busInfo.busSpeed = cni->getBaudRate();
  strcpy( busInfo.nodeName[nodeImagesCount-1], cni->getName());
  return cni;
}

void Coman::driverSetup()
{
  int i;

  if(nodeImagesCount == 0) return;

  baudRate =  nodeImages[0]->getBaudRate();
    
  for( i = 1; i < nodeImagesCount; i++ )
    {
      if( nodeImages[i]->getBaudRate() != baudRate )
        {
          Lg << ERROR << "--Error: node: " << nodeImages[i]->getName() << " baud rate: "  << nodeImages[i]->getBaudRate() 
             << ", expected: " << baudRate << endl;
        }
    }
  canDrv->initialize( baudRate );
}
/*
int Coman::eqmHandler(char *deviceName, char *property, TINE_ARGS)
{

  if( server == NULL ) return non_existent_elem;

  TineDevice **device = server->getDevices();  
  for( int i = 0; i < server->getDeviceCount(); i++ )
    {
      if( !strcmp( device[i]->getName(), deviceName )) return device[i]->propertyHandler( property, dout, din, access );
    }
  Lg << "Coman::eqmHandler called for unknown device:" << deviceName << endl;
  return non_existent_elem;
}
*/
void Coman::addProperty( char * propName,DTYPE *out, DTYPE *in, int acc, char * dsc, 
                         int (Coman::*h)( TINE_ARGS ))                                 
{
  strcpy( propertyHandlerList[propertyCounter].name, propName);
  propertyHandlerList[propertyCounter].handler = h;
  TineDevice::addProperty(propName, out, in, acc, dsc, NULL);
} 

int Coman::propertyHandler( char *property, TINE_ARGS )
{
  for( int i = 0; i < propertyCounter; i++ )        
    {
      if( !strcmp( property, propertyHandlerList[i].name) ) return ( this->*propertyHandlerList[i].handler)( dout, din, access ); 
    }

  return 0;
}

int Coman::busInfoHandler(TINE_ARGS)
{
  if( dout->dFormat != CF_STRUCT ) return illegal_format;
  if( dout->dArrayLength < 1 )
    {
      Lg << DBG << "busInfohandler: wrong data size = " <<  dout->dArrayLength << endl;
      return illegal_data_size;
    }
  for( int i = 0; i < nodeImagesCount; i++ ) busInfo.nodeState[i] = nodeImages[i]->getNodeState();
  busInfo.syncState = syncProducer->isEnabled() ? 1 : 0;
  busInfo.timeProducerState = timeProducer->isEnabled() ? 1 : 0;
  strcpy( busInfo.ticomVersion, getVersion() );
  strcpy( busInfo.ticomStartTime, getStartTime() );
  memcpy( dout->data.vptr, &busInfo, sizeof( BUS_INFO ) ); 
  return 0;
}

int Coman::getBusNameHandler(TINE_ARGS)
{
  if( dout->dFormat != CF_TEXT) return illegal_format;
  if( dout->dArrayLength < 33 )
    {
      Lg << DBG << "getBusNameHandler: wrong data size = " <<  dout->dArrayLength << endl;
      return illegal_data_size;
    }
  strcpy( dout->data.cptr, getBusName() ); 
  return 0;
}

int Coman::stopAllNodesHandler(TINE_ARGS)
{
  if( !( access & CA_WRITE)) return illegal_read_write;
  
  return sendNMT( STOP_NODE ) ? 600 : 0; 
}

int Coman::operationalAllNodesHandler(TINE_ARGS)
{
  if( !( access & CA_WRITE) ) return illegal_read_write;
  
  return sendNMT( START_NODE ) ? 600 : 0; 
}

int Coman::preoperationalAllNodesHandler(TINE_ARGS)
{
  if(!( access & CA_WRITE) ) return illegal_read_write;
  
  return sendNMT( ENTER_PREOPERATIONAL ) ? 600 : 0; 
}

int Coman::resetAllNodesHandler(TINE_ARGS)
{
  if( !( access & CA_WRITE) ) return illegal_read_write;
  
  return sendNMT( RESET_NODE ) ? 600 : 0; 
}

int Coman::sendNMT( int state )
{
  char bytes[2];
  bytes[0] = state;
  bytes[1] = 0;  // send to all nodes
  CanMessage m( 0, bytes, 2 );  
  return  cmc->submit(m); 
}

int Coman::getUninitializedDeviceCount()
{
    int counter = 0;
    for( int i = 0; i < this->getNodesCount(); i++ )
    {
	if( !getNodeImage(i)->isInitialized() ) ++counter;	
    }
    return counter;
}

void Coman::dump()
{
    Lg << INFO << "----COMAN DUMP------------------------------------" << endl;
    Lg << INFO << "Can driver: " << canDrv->getDevice() << endl;
    Lg << INFO << "Number of nodes = " << nodeImagesCount << endl;
    for( int i = 0; i < nodeImagesCount; i++ )
	Lg << INFO << "Node id = 0x" << hex << nodeImages[i]->getNodeId() << dec << endl;
    Lg << INFO << "--------------------------------------------------" << endl;
}
int Coman::consoleRequest( int argc, char * argv[] )
{
 /*   
    
  char *words[10];
  int nwords, node;
  int i;

  cout << "consoleRequest: argc = " << argc << endl;

  if( argc < 1 )
    {
      cout << endl << "Dump of Coman:" << endl;
      for( i = 0; i < LxComanA->nodeImagesCount; i++ )
        cout << "Node[" << i << "], nodeId = 0x" << hex << LxComanA->nodeImages[i]->getNodeId() << dec << ", name  = " << LxComanA->nodeImages[i]->getName() << endl;
      cout << endl;
      return 0;
    }

  
  for( i = 0; i < argc; i++ ) cout << "argv[" << i << "] = " << argv[i] << endl;

  if( (nwords = crack( argv[0], words, "/\\", 10 )) < 2 ) 
    {
      if( !strcmp(words[0], "sync"))
        {          
          LxComanA->syncProducer->dump();
          return 0;
        } 

      if( ( node = strtol( words[0], NULL, 0 ) ) > 0 )
        {
          for( i = 0; i < LxComanA->nodeImagesCount; i++ )
            {
              if( LxComanA->nodeImages[i]->getNodeId() == node )
                {
                  LxComanA->nodeImages[i]->dump();
                  return 0;
                }
            } 
        }
      else
        {
          for( i = 0; i < LxComanA->nodeImagesCount; i++ )
            {
              if( !strcmp(words[0], LxComanA->nodeImages[i]->getName()) )
                {
                  LxComanA->nodeImages[i]->dump();
                  return 0;
                }
            }
        }
    }

   if( ( node = strtol( words[0], NULL, 0 ) ) > 0 )
     {
       for( i = 0; i < LxComanA->nodeImagesCount; i++ )
         {
           if( LxComanA->nodeImages[i]->getNodeId() == node )
             {
               LxComanA->nodeImages[i]->dump(nwords-1, &words[1]);
               return 0;
             }
         } 
     }
   else
     { 
       for( i = 0; i < LxComanA->nodeImagesCount; i++ )
         {
           if( !strcmp(words[0], LxComanA->nodeImages[i]->getName()) )
             {
               LxComanA->nodeImages[i]->dump( nwords-1, &words[1] );
               return 0;
             }
         }
     }
 */
  return 0;
}

