

#ifndef __COMAN_H__
#define __COMAN_H__

#include "loggerClass/log.h"
#include "canMessageCollector.h"
#include "canMessageDispatcher.h"
#include "canDriver.h"
#include "canDriverEsd.h"
#include "canNodeImage.h"
#include "sync.h"
#include "timeProducer.h"
#include "tineServer.h"
#include "tools.h"
#include "version.h"

extern Log Lg;

const int MAX_NODE_IMAGE_COUNT  = 127;


typedef struct
{
  long  nNodes;
  char  nodeName[MAX_NODE_IMAGE_COUNT][33];
  char  busName[33];
  long  nodeState[MAX_NODE_IMAGE_COUNT];
  long  syncState;
  long  timeProducerState;
  long  busSpeed;
  char  ticomVersion[64];
  char  ticomStartTime[64];
} BUS_INFO;


class Coman : public TineDevice //, public DefaultTineHandler
{
private:
  struct ph
  {
    char name[33];
    int (Coman::*handler)( TINE_ARGS );
  }  propertyHandlerList[MAX_PROPERTY];

  int baudRate;
  int nodeImagesCount;
  BUS_INFO busInfo; 
  CanNodeImage * nodeImages[MAX_NODE_IMAGE_COUNT];  
  CanDriver *canDrv;
  CanMessageCollector *cmc;
  CanMessageDispatcher *cmd;
  SyncProducer * syncProducer;
  TimeProducer * timeProducer;
  TineServer * server;
  void construct(TineServer * srv, int canDevNo);
  void buildProperties();
  void addProperty( char * propName, DTYPE * out, DTYPE * in, int acc, char * dsc, int (Coman::*h)( TINE_ARGS ));
  int propertyHandler( char *property, TINE_ARGS );

public: 
 
  Coman(TineServer* srv);
  Coman(TineServer* srv, int canInterfaceNumber);
  CanNodeImage* add(char *dcfName);
  CanNodeImage* add(char *dcfName, char *nodeName, int nodeId);
  CanNodeImage* add(CanNodeImage * cni);
  void driverSetup();
  int sendNMT( int state );
  void dump();
  inline int getNodesCount() { return nodeImagesCount; }
  inline SyncProducer * getSyncProducer() { return syncProducer; }
  inline TimeProducer * getTimeProducer() { return timeProducer; }
  inline CanNodeImage * getNodeImage( int nodeIndex ) { return nodeImages[nodeIndex]; }
  inline CanDriver * getCanDriver() { return canDrv; }
  inline CanMessageCollector * getCanMessageCollector() { return cmc; }
  inline CanMessageDispatcher * getCanMessageDispatcher() { return cmd; }
  inline CanNodeImage ** getNodeImages() { return nodeImages; }
  inline void setBusName( char *bname ) { strncpy(busInfo.busName, bname, 32); busInfo.busName[32] ='\0'; }
  inline char * getBusName() { return busInfo.busName; }
  int busInfoHandler(TINE_ARGS);
  int stopAllNodesHandler(TINE_ARGS);
  int operationalAllNodesHandler(TINE_ARGS);
  int preoperationalAllNodesHandler(TINE_ARGS);
  int resetAllNodesHandler(TINE_ARGS);
  int getBusNameHandler(TINE_ARGS);
  int getUninitializedDeviceCount();
  //int eqmHandler(char *deviceName, char *property, TINE_ARGS);
  static int consoleRequest( int argc, char * argv[] );
};
#endif
