
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "loggerClass/log.h"
#include "dirList.h"
#include "dbfile.h"
#include "configurator.h"
#include "canDriver.h"
#include "tineServer.h"


extern Log Lg;

Configurator::Configurator()
{
    construct();
}

Configurator::Configurator(char* filename)
{
    construct(filename);
}

Configurator::Configurator( int argc, char** argv )
{
    construct();
    parse( argc, argv );
}

void Configurator::construct()
{
  construct("none");
}
void Configurator::construct(char* fname) 
{
    
    memset( &tineFecConfig, '\0', sizeof( TineFecConfig ) );
    tineFecConfig.portOffset = -1;  // -1 == 'unitialised'
    
    busCount = 0;
    csvUsed = 0;
    if (!strcmp(fname,"none")) memset(fileName,'\0',MAX_FN_LEN); // SWH
    else 
    {  
      strncpy(fileName,fname,MAX_FN_LEN-1);  // SWH, don't set csvUsed yet!
    }                  
    memset(nodeName,'\0', MAX_BUS_COUNT * 127 * 32);
    for(int i=0; i < MAX_BUS_COUNT; i++ )
    {
        busName[i][0] = '\0';    
        cdiBusNumber[i] = 0;
        serverName[i][0] = '\0';
        nodeCount[i] = 0;        
        canInterfaceNumber[i] = 0;
        debugLevel = 0;
        //csvUsed = 0;
    }           
}

void Configurator::parse( int argc, char ** argv )
{
    int i;
    int prevLogLevel = Lg.setWarningLevel();
    strcpy(serverName[0], "TICOM" );
    nodeCount[0] = 0;
    busCount = 1;
    canInterfaceNumber[0] = PCAN_USB;
    debugLevel = MSG_WARNING;
    esdDriver = 0;
   
    for( i = 1; i < argc; i++ )
       {
	 if( !strcmp( "-h", argv[i] ) ) help();
         if( !strcmp( "-tine.s", argv[i] ) )
           {
             if( i+1 < argc ) strcpy( serverName[0],argv[++i] );
             else
               { 
                 printf( "\nServer name is missing. Use 'h' for help\n\n" );
                 exit(1);
               }
             continue;
           }        
	  if( !strcmp( "-l", argv[i] ) )
           {
             if( i+1 < argc ) 
	     {
		 strncpy( busName[0],argv[++i], 32 );
		 busName[0][32] = '\0';
	     }
             else
               { 
                 printf( "\nLine name is missing. Use 'h' for help\n\n" );
                 exit(1);
               }
             continue;
           }      
         if( !strcmp( "-n", argv[i] ))
           {
             if( i+1 < argc ) canInterfaceNumber[0] = atoi(argv[++i]);
             else
               { 
                 printf( "\nCan interface number is missing. Use 'h' for help\n\n" );
                 exit(1);
               }
             continue;
           }         
         if( !strcmp( "-d", argv[i] ))
           {
             if( i+1 < argc ) debugLevel = atoi(argv[++i]);
             else
               { 
                 printf( "\nDebug level is missing. Assumed DebugLevel = 1\n\n" );
                 debugLevel = 1;
               }
             continue;
          }            
          if( !strcmp( "-esd", argv[i]))
          {
             esdDriver = 1;
             continue;
          }                   
          if( !strcmp( "-tine.p", argv[i]))
          {
             if( i+1 < argc ) 
             {
                 int po = atoi(argv[++i]);
                 if( po < 0 ) 
                 {
                     printf( "\nTINE port offset must not be a negative number\n");
                     exit(1);
                 }
                 tineFecConfig.portOffset = po;
             }
             else 
             {
                 printf( "\nTINE port offset value is missing.\n\n" );
                 exit(1);
             }                 
             continue;
          }  
         if( !strcmp( "-tine.f", argv[i] ) )
           {
             if( i+1 < argc ) strcpy( tineFecConfig.fecName, argv[++i] );
             else
               { 
                 printf( "\nTINE fec name is missing. Use 'h' for help\n\n" );
                 exit(1);
               }
             continue;
           } 
         if( !strcmp( "-tine.c", argv[i] ) )
           {
             if( i+1 < argc ) strcpy( tineFecConfig.context, argv[++i] );
             else
               { 
                 printf( "\nTINE context name is missing. Use 'h' for help\n\n" );
                 exit(1);
               }
             continue;
           }   
         if( !strcmp( "-tine.d", argv[i] ) )
           {
             if( i+1 < argc ) strcpy( tineFecConfig.description, argv[++i] );
             else
               { 
                 printf( "\nTINE description string is missing. Use 'h' for help\n\n" );
                 exit(1);
               }
             continue;
           }          
         if( !strcmp( "-tine.o", argv[i] ) )
           {
             if( i+1 < argc ) strcpy( tineFecConfig.operatingSystem, argv[++i] );
             else
               { 
                 printf( "\nTINE operating system name is missing. Use 'h' for help\n\n" );
                 exit(1);
               }
             continue;
           }                                     
         if( !strcmp( "-tine.l", argv[i] ) )
           {
             if( i+1 < argc ) strcpy( tineFecConfig.location, argv[++i] );
             else
               { 
                 printf( "\nTINE location string is missing. Use 'h' for help\n\n" );
                 exit(1);
               }
             continue;
           } 
         if( !strcmp( "-tine.h", argv[i] ) )
           {
             if( i+1 < argc ) strcpy( tineFecConfig.hardware, argv[++i] );
             else
               { 
                 printf( "\nTINE hardware parameter is missing. Use 'h' for help\n\n" );
                 exit(1);
               }
             continue;
           } 
         if( !strcmp( "-tine.r", argv[i] ) )
           {
             if( i+1 < argc ) strcpy( tineFecConfig.responsible, argv[++i] );
             else
               { 
                 printf( "\nTINE 'responsible' is missing. Use 'h' for help\n\n" );
                 exit(1);
               }
             continue;
           } 
                           
         
          if( !strcmp( "-csv", argv[i]))
          {
             if( i+1 < argc ) 
             {
                 csvUsed = 1;
                 parseCSV(argv[++i]);                 
             }
             else printf( "\nCSV file name is missing. Option ignored.\n\n" );                 
             continue;
          }
       }
    if (!csvUsed && strlen(fileName)) parseCSV();  // SWH  fileName from constructor, sets csvUsed!
    else if( !csvUsed )
    {
        // no csv used, scan current directory for DCF files
        DirList dirList( ".", ".dcf" );
        nodeCount[0] = 0;
        if( dirList.getListLen() )
          {
            dirList.goToFirst();
            do strcpy(nodeDCFname[0][nodeCount[0]++], dirList.getEntry());
            while( dirList.goToNext() );
           }	
    }
    
    Lg.setLoggingLevel(prevLogLevel);
}

void Configurator::parseCSV(char* fname) { // SWH  fileName from -csv
  strncpy(fileName,fname,MAX_FN_LEN -1); 
  parseCSV();
}

void Configurator::parseCSV()  // SWH  
{
   DbCsv* db = new DbCsv(257,',');
   ifstream dbin( fileName );
   if (!dbin.is_open()) { Lg << FATAL  << "Configurator: Open Fails for file " << fileName << endl; exit(1); }
   int nvals, ncols;
   int currentBusIndex;
   char * bus;
   //  21.01.2009: char * server;
   char * nodeDcf;  // SWH   
   int  canNumber;
   int  cdiNumber;
  
   csvUsed = 1;   // SWH: case filename from Constructor

   if( (ncols = db->initHeader(&dbin)) < 1 ) 
   { 
       Lg << ERROR << "--Error: reading " << fileName << ", number of columns = " << ncols << endl; 
       exit(1); 
   }

   busCount = 0;
   while( !dbin.eof() ) 
   {
     nvals = db->getNextLine( &dbin );
     if ( nvals < 1) break; // EOF
     
     bus = db->stringv("LINENAME");
     //server = db->stringv("SRVNAME");
     canNumber = db->intv("IFACEID"); 
     cdiNumber = db->intv("CDINUM");     
     nodeDcf = db->stringv("NODEDCF");  // SWH
     if (!strcmp(nodeDcf,"none")) Lg << INFO << db->stringv("NODENAME") << " : No EDS"  << endl; 
     else
     {
	Lg << INFO << db->stringv("NODENAME") << " : NodeEDS " << nodeDcf << endl;
     
	 if( !busNameKnown( bus, &currentBusIndex ) )  // SWH assumes that CRATE comes first in list
	 {   
	     currentBusIndex = busCount;
	     if( ++busCount > MAX_BUS_COUNT)
	     {
		Lg << ERROR << "--Error: too many CAN buses (" << busCount << ") found in the csv file: " << fileName << endl;
		exit(1);
	     }
	     strcpy(busName[currentBusIndex], bus);
	     strcpy(serverName[currentBusIndex],  bus );  // 21.01.2009: server);
	     cdiBusNumber[currentBusIndex] = cdiNumber;
	     canInterfaceNumber[currentBusIndex] = canNumber;
	     nodeCount[currentBusIndex]=1;
	 }
	 else
	 {
	     // check consistency
	     int error = 0;
	     if( strcmp(busName[currentBusIndex], bus) ) error = 1;
	     if( strcmp(serverName[currentBusIndex], bus) ) error = 1;  // 21.01.2009: server) ) error = 1;
	     if( cdiBusNumber[currentBusIndex] != cdiNumber ) error = 1; 
	     if( canInterfaceNumber[currentBusIndex] != canNumber ) error = 1;
	     if( error )
	     {
		 Lg << FATAL << "--Error: inconsistency in the config file: " << fileName << endl;
		 exit(1);
	     }              
	    ++nodeCount[currentBusIndex];
	 }
	 strcpy(nodeName[currentBusIndex][nodeCount[currentBusIndex]-1], db->stringv("NODENAME"));
	 strcpy(nodeDCFname[currentBusIndex][nodeCount[currentBusIndex]-1], db->stringv("NODEDCF"));
	 nodeId[currentBusIndex][nodeCount[currentBusIndex]-1] = db->intv("NODEID");     
     }	 
   }     
   delete db;
}


int Configurator::busNameKnown(char * name, int * index)
{
    for(int i = 0; i < MAX_BUS_COUNT; i++)
    {
        if( !strcmp(name, busName[i])) 
        {
            *index = i;
            return 1;
        }
    }
    
    return 0;
}

void Configurator::help( void )
{
  printf( "\nusage: ticom [options...]\n" );
  printf( "\tuse csv option exclusively with options -n, -s" );
  printf( "\noptions:\n" );
  printf( "\t-csv <csv configuartion file name> : use a CSV configuartion file\n");
  printf( "\t-esd               : use an ESD CAN driver rather than a PEAK CAN driver (default)\n" );
  printf( "\t-n <number>        : PEAK can device number (%d = PCAN_USB, when omitted) or ESD net number\n", PCAN_USB );
  printf( "\t-d <number>        : run in debug mode with given debug level \n" );  
  printf( "\t-l <name>		: assign <name> to the CAN bus\n" );
  printf( "\t-tine.s <name>     : register as TINE server <name> \n" );
  printf( "\t-tine.p <number>   : use <number> as a port offset for TINE FEC\n" );
  printf( "\t-tine.f <name>     : use <name> as a TINE FEC name\n" );
  printf( "\t-tine.c <name>     : use <name> as a TINE FEC context\n" );
  printf( "\t-tine.d <desc>     : use <desc> as a TINE FEC description\n" );
  printf( "\t-tine.l <location> : location of a TINE FEC\n" );
  printf( "\t-tine.h <hardware> : hardware used by the TINE FEC\n" );
  printf( "\t-tine.r <name>     : name of a responsible person of the TINE FEC\n" );
  printf( "\t-h                 : print this help\n" );
  exit(0);
}

