// 
// File:   configurator.h
// Author: karol
//
// Created on November 14, 2007, 3:20 PM
//

#ifndef __CONFIGURATOR_H__
#define	__CONFIGURATOR_H__

#include "string.h"

#define MAX_BUS_COUNT 8
#define MAX_FN_LEN 64

class TineFecConfig
{   
    public:
        short portOffset;
        char fecName[17];
        char context[17];
        char description[33];
        char operatingSystem[33];
        char location[33];
        char hardware[33];
        char responsible[33];
};

class Configurator
{
    private:
        int  busCount;
        char fileName[MAX_FN_LEN];  // SWH
        char busName[MAX_BUS_COUNT][32];
        int  cdiBusNumber[MAX_BUS_COUNT];
        char serverName[MAX_BUS_COUNT][32];
        int  nodeCount[MAX_BUS_COUNT];        
        char nodeName[MAX_BUS_COUNT][127][32];        
        char nodeDCFname[MAX_BUS_COUNT][127][128];        
        int  nodeId[MAX_BUS_COUNT][127];           
        int  canInterfaceNumber[MAX_BUS_COUNT];
        int  debugLevel;
        int  csvUsed;
        int  esdDriver;
        void parseCSV(void);  // SWH
        void parseCSV(char * filename);
        int  busNameKnown(char * name, int * index);
        
        TineFecConfig tineFecConfig;
       
        void construct();
        void construct(char * filename);
        
    public:
        Configurator();
        Configurator(char * fileName);  // SWH for default csv file 
        Configurator( int argc, char ** argv );        
        void parse( int argc, char ** argv );
        void help();
        inline void   setCanBusCount(int bc) { busCount = bc; }
        inline void   setCanInterfaceID(int canBusNo, int canIfNo) { canInterfaceNumber[canBusNo] = canIfNo; }
        inline void   setTineServerName(int canBusNo, char * name ) { strncpy(serverName[canBusNo], name, 31); serverName[canBusNo][31] = '\0'; }
        inline void   setNodeCount(int canBusNo, int nc) { nodeCount[canBusNo] = nc; }
        inline void   setNodeName(int canBusNo, int deviceNo, char * nn) { strncpy(nodeName[canBusNo][deviceNo], nn, 31); nodeName[canBusNo][deviceNo][31] = '\0'; }
        inline void   setNodeDCFname(int canBusNo, int deviceNo, char *dcf) { strncpy( nodeDCFname[canBusNo][deviceNo], dcf,127 ); nodeDCFname[canBusNo][deviceNo][127] = '\0'; }
        inline void   setNodeId(int canBusNo, int deviceNo, short nId) { nodeId[canBusNo][deviceNo] = nId; }
        inline void   setLoggingLevel(int ll) { debugLevel = ll; }
        inline void   useEsdDriver(int useEsd) { esdDriver = useEsd; }                
        inline TineFecConfig * getTineFecConfig() { return &tineFecConfig; }
        inline int    getCanBusCount() { return busCount; }
	inline char * getBusName(int idx) { return idx < busCount ? busName[idx] : (char*)"not existing bus"; }
        inline int    getCanInterfaceID(int canBusNo) { return  canInterfaceNumber[canBusNo]; }
        inline char * getTineServerName(int canBusNo ) { return serverName[canBusNo]; }
        inline int    getNodeCount(int canBusNo) { return nodeCount[canBusNo]; }
        inline char * getNodeName(int canBusNo, int deviceNo) { return nodeName[canBusNo][deviceNo]; }
        inline char * getNodeDCFname(int canBusNo, int deviceNo) { return nodeDCFname[canBusNo][deviceNo]; }
        inline int    getNodeId(int canBusNo, int deviceNo) { return nodeId[canBusNo][deviceNo]; }
        inline char * getFileName() { return fileName; }  // SWH
        inline int    getLoggingLevel() { return debugLevel; }
        inline int    isCsvUsed() { return csvUsed; }
        inline int    isEsdDriver() { return esdDriver; }
};

#endif	/* __CONFIGURATOR_H__ */

