//: dbfile.cpp:  methods for csv file read
#include "dbfile.h"

DbCsv::DbCsv(int primesz, char tok) {
  token = tok;
  primesize = primesz;
  iniHash = new CoalHashTableNI(primesize);
}
DbCsv::~DbCsv() {
  delete iniHash; // ?? check destructor operation
}
// return #entries for ok, 0 for eof, -1 for error
int DbCsv::getNextLine(ifstream *dbin) {
  char iobuf[MAX_BUFFER_SIZE];
  do {
     dbin->get(iobuf,sizeof(iobuf),'\n'); dbin->get();
  } while (iobuf[0] == '#');
  return readLine(iobuf);
}
// store line internally, store data start positions and
// replace tokens with null term.
// return # of entries found, EOF for only 0 or null entry
// what does this do with other null entries ?? ie ,,,
int DbCsv::readLine(char* dline) {
  char *start,*stop;
  int n, nchar=0;
  if (dline[0] == '#') return 0;
  strncpy(buf,dline,MAX_BUFFER_SIZE-1); // copy not needed ??
  start = buf;
  for (int i=0; i<tsize; i++) {
    stop = strchrnul(start,token);
    pos[i] = (int)(start - buf);
    if (*stop == '\0') {
      if (!i) { // case blank line at end => return EOF
        n = stop - start;
	if (!n) return 0;
        for (int j=0; j<n; j++) if (start[j] != ' ') nchar++; // found a character
        if (!nchar) return 0; // or didn't
      }
      return i+1;
    }
    *stop = '\0';
    start = stop+1;
  }
  return -1; // can we reach this?
}
// return null-term string with ith value (it could be 0 length)
char* DbCsv::stringv(char* name) {
  char *ic;
  int m,n=0;
  int ind = iniHash->GetIndex(name);
  if (ind<0) return 0;
  ic =  &(buf[pos[ind]]);
  // trim leading/trailing blanks!
  while (ic[n] == 32) n++;
  ic = ic+n;
  n=1;
  m = strlen(ic);
  if (m) { while (ic[m-n] == 32) { ic[m-n] = 0; n++; } }
  return ic;
}
int DbCsv::intv(char* name) { // no error checking!
  int ind = iniHash->GetIndex(name);
  if (ind<0) return 0;
  return  atoi(&(buf[pos[ind]]));
}
float DbCsv::floatv(char* name) { // no error checking!
  int ind = iniHash->GetIndex(name);
  if (ind<0) return 0;
  return (float)atof( &(buf[pos[ind]]));
}
// encapsulate dbin operations for header init
int DbCsv::initHeader(ifstream* dbin) {
  char iobuf[MAX_BUFFER_SIZE];
  int rc;
  do {
    dbin->get(iobuf,sizeof(iobuf),'\n'); dbin->get();
  } while (!(rc = readHeader(iobuf))); // throw away lines with leading #
  return rc;
}
// Read Header Line: Parse Header, put NAME/ColNumber pairs in HashTable
// NAMEs are converted to upper case with leading/trailing blanks removed
int DbCsv::readHeader(char* hline) {
  char* start, *stop;
  NameInd nind;
  int n;
  if ('#' == hline[0]) return 0;
  stop = start = hline;
  for (int i=0; i<primesize; i++) {
    if (*stop == '\0') { cout << endl;  return tsize; }
    stop = strchrnul(start,token);
    n = stop - start;
    while (start[0] == ' ') { start++; n--; }  // leading ws?
    strncpy(buf,start,n);
    buf[n] = '\0';
    while (buf[n-1] == ' ') { n--; buf[n] = '\0'; }  // trailing ws?
    if (!n) { cout << endl;  return (*stop == '\0')? tsize : -1;  }   // null ok at end of line
    for (int j=0; j<n; j++) buf[j] = toupper(buf[j]);  // to upper case
    cout << buf << ":"; // print-out, close spacing to check for ws
    nind.set(buf,i);
    iniHash->Insert(nind);
    start = stop+1;
    tsize = i+1;
  }
  cout << endl;
  return -1;
}
