
#include <string.h>
#include <errno.h>
#include "edsParser.h"



EDSparser::EDSparser()
{
  construct( NULL, "" );
}

EDSparser::EDSparser( void * owner, char *fname )
{
  construct( owner, fname );
}

void EDSparser::construct(  void * owner, char *fname )
{ 
  strncpy(filename, fname, MAX_FILE_NAME );
  filename[MAX_FILE_NAME] = '\0';
  parserCaller = owner;
  currentSectionName[0] = '\0';
  currentIndex = -1;
  currentSubindex = -1;
}

int EDSparser::parse()
{

  FILE * h = fopen( filename, "r" );
  char * words[MAX_WORDS];
  char line[MAX_LINE_LENGTH +1];
  int nwords;
  int lineCnt = 0;
  
  if( h == NULL )
  {
    Lg << ERROR << "--Error: EDSparser::parse(): can not open a file: " << filename <<", errno = " << errno << endl;
    return errno;
  }

  while( !feof(h))
  {
    if( fgets(line, MAX_LINE_LENGTH, h ) != NULL )
      {       
        ++lineCnt;
        nwords = crack( trimCRLF(line), words, " =", 2 ); //  MAX_WORDS );      
        if( nwords > 0 ) 
          {
            if( *words[0] != ';' ) // skip comments
              {
                if( *words[0] == '[' )
                  {                                          
                    char *pos;
                    // find and remove the closing bracket ']'
                    if( ( pos = strchr(words[0], ']' )) != NULL ) *pos = '\0';
                    strcpy( currentSectionName, words[0] + 1 );  // +1 : skip the opening bracket '['  
                    toUpperString( currentSectionName );
                    if( isdigit(currentSectionName[0])) // asssume a section starting with a digit is a description of OD entry, section name is giving index and subindex
                      {  
                        if( (pos = strstr( currentSectionName, "OBJECTLINKS" )) != NULL )
                          {
                            Lg << FATAL << "---Fatal: <index>OBJECTLINKS not supported by the EDS parser, exiting..." << endl;
                            exit(1);
                          }
                        if( (pos = strstr( currentSectionName, "NAME" )) != NULL )
                          {
                            Lg << FATAL << "---Fatal: <index>NAME not supported by the EDS parser, exiting..." << endl;
                            exit(1);
                          }
                        
                        if( (pos = strstr( currentSectionName, "SUB" )) == NULL ) currentSubindex = -1;                           
                        else
                          {
                            *pos = '\0';
                            pos+=3;
                            currentSubindex = strtol( pos, NULL, 16 );
                          }
                        currentIndex = strtol( currentSectionName, NULL, 16 );
                      }
                    else currentIndex = currentSubindex = -1;  // no valid index, subindex
                  }
                else
                  {
                    if( nwords > 1 )
                      {                      
                        words[1] = trim( words[1] ); // remove spacess around argument
                      }
                    else words[1] = "";
                    
                    if( currentIndex > -1 ) objectDictionaryCallback( parserCaller, currentIndex, currentSubindex, toUpperString(words[0]), words[1], lineCnt );
                    else sectionCallback( parserCaller, currentSectionName, toUpperString(words[0]) , words[1] ); 
                  }               
              }
          }
      }
  }
  fclose(h);
  sectionCallback( parserCaller, "", "", "" );  // force the temporary buffer flush
  return 0;
}

int EDSparser::parse( void * owner, char *fname, void (* sectionCb )( void *, char *, char *, char *), void (*odCb)(void *, int, int, char *, char *, int )  )
{
   strncpy(filename, fname, MAX_FILE_NAME );
   filename[MAX_FILE_NAME] = '\0';

   parserCaller = owner;

   sectionCallback = sectionCb;
   objectDictionaryCallback = odCb;

   return parse();
}


