/* 
 * File:   flashProgrammer.h
 * Author: karol
 *
 * Created on November 27, 2008, 5:39 PM
 */

#ifndef __FLASHPROGRAMMER_H__
#define	__FLASHPROGRAMMER_H__

#include "canNodeImage.h"
#include "motorolaSrecordReader.h"

class FlashProgrammer : public TimeSlice
{
    private:

        static const int UNLOCK_BRENNMODE = 1;
        static const int UNLOCK_WAIT_FOR_RESPONSE = 2;
        static const int WAIT_FOR_BOOT_UP_STATE = 3;
        static const int WAIT_FOR_PREOPERATIONAL_STATE = 4;
        static const int INIT_DOWNLOAD = 11;
        static const int INIT_WAIT_FOR_RESPONSE = 12;
        static const int SEND_DATA = 13;
        static const int SEND_DATA_WAIT_FOR_RESPONSE = 14;
        static const int DATA_SENT = 15;
        static const int ERASE_AND_PROGRAMMING = 16;
        static const int DATA_PROGRAMMED = 17;
        static const int CHECK_PROGRAMMING_STATUS = 18;
        static const int WAIT_FOR_STATUS = 19;
        static const int CALCULATE_PARITY = 20;
        static const int WAIT_FOR_CALCULATE_PARITY = 21;
        static const int CHECK_RAM_PARITY = 22;
        static const int WAIT_FOR_RAM_PARITY = 23;
        static const int CHECK_FOR_FLASH_PARITY = 24;
        static const int WAIT_FOR_FLASH_PARITY = 25;

        static const int UNLOCK_MAIN_MODE = 30;
        static const int JUMP_TO_MAIN_MODE = 31;

        static const int SINGLE_STEP_SEND_DATA = 64;
        static const int SINGLE_STEP_SIZE_WRITTEN = 65;
        static const int SINGLE_STEP_ERASE_AND_REPROGRAM = 66;
        static const int SINGLE_STEP_DATA_PROGRAMMED = 67;
        static const int SINGLE_STEP_CHECK_PROGRAMMING_STATUS = 68;
        static const int SINGLE_STEP_WAIT_FOR_STATUS = 69;

        static const int FINISHED = 128;
        CanNodeImage * owner;
        SDOclient * sdoClient;
        HrTimer *autoResetTimer;
        MotorolaSrecordReader * motorolaSrecordReader;
        int flashParity;
        int ramParity;
        int fileParity;
        char *  binaryDataBuffer;
        int controllerStatus;
        int error;        
        int dataSize;
        int state;
        int status;
        void deleteBinaryDataBuffer();

        void handleUnlockBrennMode();
        void handleUnlockWaitForResponse();
        void handleWaitForBootupState();
        void handleWaitForPreoperationalState();
        void handleInitDownload();
        void handleInitWaitForResponse();
        void handleSendData();
        void handleSendDataWaitForResponse();
        void handleDataSent();
        void handleEraseAndProgramming();
        void handleDataProgrammed();
        void handleCheckProgrammingStatus();
        void handleWaitForStatus();
        void handleCalculateParity();
        void handleWaitForCalculateParity();
        void handleCheckRamParity();
        void handleWaitForRamParity();
        void handleCheckFlashParity();
        void handleWaitForFlashParity();
        void handleUnlockMainMode();
        void handleJumpToMainMode();
        void handleSingleStepSendData();
        void handleSingleStepSizeWritten();
        void handleSingleStepEraseAndReprogram();       
        void handleSingleStepDataProgrammed();
        void handleSingleStepCheckProgrammingStatus();
        void handleSingleStepWaitForStatus();

        int  checkSdoError();
        int unlock(int nextState);
        int  checkSdoAvailable(int idx, int subidx);
        
    public:
        static const int OK = 0;
        static const int BUSY = 1;
        static const int NO_SDO_CHANNEL = 2;
        static const int DATA_CORRUPTED = 3;        

        static const int IDLE = 0;
        static const int SWITCHING_TO_PROGRAMMING_MODE = 1;
        static const int SENDING_DATA = 2;
        static const int REPROGRAMMING = 3;
        static const int CHECKING_CONTROL_SUMS = 4;
        static const int BOOTING = 5;

        FlashProgrammer(CanNodeImage * owner);
       ~FlashProgrammer();
	int available();
        inline int isActive() { return state != IDLE; }
	int start( char * fileName );
        int start( char * fileName, int initialState );
        int done();
        inline int getError() { return error; }
        char * getAbortDescription();
        char * getErrorDescription();
        void reset();
	inline int getState() { return state; }
        inline int getStatus() { return status; }
        inline void setStatus( int s) { status = s; }
        inline int getProgress() { return sdoClient->getProgress(); }
        inline int getFileParity() { return fileParity; }
        inline int getRamParity() { return ramParity; }
        inline int getFlashParity() { return flashParity; }
        int  sendData( char * fileName );
        int unlockForErasingAndProgramming();
        void activate();
        inline int * getFlashParityPtr() { return &flashParity; }
        inline int * getRamParityPtr() { return &ramParity; }
};

#endif	/* __FLASHPROGRAMMER_H__ */

