#include "heartBeat.h"
#include "canNodeImage.h"

HeartBeatConsumer::HeartBeatConsumer(CanNodeImage * o, long period )
{
  init( o, period);
}

HeartBeatConsumer::HeartBeatConsumer(CanNodeImage * o)
{ 
  init(o, 2000);
}

char * HeartBeatConsumer::getStateName(int st)
{
    switch(st)
    {
	case BOOT_UP:	     return "BOOTUP";
	case INIT:	     return "INIT (internal state)";
	case STOPPED:	     return "STOPPED";
	case OPERATIONAL:    return "OPERATIONAL";
	case PREOPERATIONAL: return "PREOPERATIONAL";		    	   
    }
    return "UNKNOWN";	    
}
void HeartBeatConsumer::init( CanNodeImage * o, long p )
{
  period = p;
  memset( stateCounter, '\0', sizeof(stateCounter) );
  memset( stateEntryTime, '\0', sizeof(stateEntryTime) );
  Timer = new HrTimer( period );
  owner = o;
  nodeState = INIT;
  stop();
}

HeartBeatConsumer::~HeartBeatConsumer()
{
  delete Timer;
}

void HeartBeatConsumer::activate()
{
  if(!running) return;
  if(Timer->expired()) 
    {
      if(!(nodeState &  HEARTBEAT_TIMEOUT)) stateEntryTime[4] = HrTimer::unixTimestamp();
      nodeState |= HEARTBEAT_TIMEOUT;  
    }
}

void HeartBeatConsumer::setNodeState(int state)
{
  
  switch(state &= 0x7F)  // remove toggling bit for Node Guard response
    {      
    case BOOT_UP:
      ++stateCounter[0]; 
      if( nodeState != state ) 
        {
	  Lg << DBG << "NMT: leaving state: " << getStateName(nodeState) << ", entering state: " << getStateName(state) << endl;
          nodeState = state; 
          stateEntryTime[0] = HrTimer::unixTimestamp();	  
        }
      Timer->restart(); 
      break;
    case PREOPERATIONAL:     
      ++stateCounter[1]; 
      if( nodeState != state ) 
        {
	  Lg << DBG << "NMT: leaving state: " << getStateName(nodeState) << ", entering state: " << getStateName(state) << endl;
          nodeState = state; 
          stateEntryTime[1] = HrTimer::unixTimestamp();
        }
      Timer->restart(); 
      break;
    case OPERATIONAL: 
      ++stateCounter[2]; 
      if( nodeState != state ) 
        {
	  Lg << DBG << "NMT: leaving state: " << getStateName(nodeState) << ", entering state: " << getStateName(state) << endl;
          nodeState = state; 
          stateEntryTime[2] = HrTimer::unixTimestamp();
        }
      Timer->restart(); 
      break;
    case STOPPED: 
      ++stateCounter[3];
      if( nodeState != state ) 
        {
	  Lg << DBG << "NMT: leaving state: " << getStateName(nodeState) << ", entering state: " << getStateName(state) << endl;
          nodeState = state; 
          stateEntryTime[3] = HrTimer::unixTimestamp();
        }
      Timer->restart(); 
      break;
    case INIT:	
      break;
    default:       
      Lg << WARNING << "-Warning: unknown NMT state: " << state << " for node 0x" << hex << owner->getNodeId() << dec << endl;
      break;
    } 
}
