#ifndef __HEARTBEAT_H__
#define __HEARTBEAT_H__


#include "loggerClass/log.h"
#include "hrtimer.h"
#include "timeSlice.h"

const int HEARTBEAT_TIMEOUT = 0x8000;


const int BOOT_UP = 0;
const int INIT = 1;  // internally used
const int PREOPERATIONAL = 127;
const int OPERATIONAL = 5;
const int STOPPED = 4;


class CanNodeImage;
extern Log Lg;

class HeartBeatConsumer : public TimeSlice
{
 private:
  long period;
  long lastUpdate;
  int nodeState;
  bool running;
  CanNodeImage * owner;
  HrTimer * Timer;
  long stateCounter[4];
  long stateEntryTime[5];
  void init( CanNodeImage * o, long p );

 public:
  HeartBeatConsumer(CanNodeImage * owner);
  HeartBeatConsumer(CanNodeImage * owner, long period);
  ~HeartBeatConsumer();
  void activate();
  inline void setPeriod( long p ) { Timer->start(period = (p == 0 ? 1000 : p) ); }
  inline void stop()  { running = false; Lg << DBG << "HeartBeat consumer stopped" << endl; }
  inline void start() { if(!running) { Timer->restart(); Lg << DBG << "HeartBeat consumer started, timer = " << Timer->getInterval() << "[ms]"<< endl; } running = true; }
  void setNodeState(int state);
  inline int getNodeState() { return nodeState; }
  long * getStateCounters() { return stateCounter; }
  long * getStateEntryTimes() { return stateEntryTime; } 
  static char * getStateName(int st);
};

#endif
