


#include "linkedList.h"


LinkedList::LinkedList()
{
	current   = NULL;
	first     = NULL;
	last      = NULL;
	nElements = 0;	
}

LinkedList::~LinkedList()
{
	LISTITEM * item;
	LISTITEM * previousItem;
	item = last;  //end of the list
	while( item )
	{
		previousItem = item->previous;
		if( item->dataBufferAllocated ) free( item->data ); 
		free( item );
		item = previousItem;
	}
}

void * LinkedList::add( void * data )
{
	LISTITEM * li = (LISTITEM*)malloc( sizeof( LISTITEM ) );

	if( li == NULL ) return NULL;

	li->dataBufferAllocated = 0;
	if( last != NULL ) 
	{
		li->previous = last;
		last->next =  li;		
	}
	else
	{
		first = li;
		li->previous = NULL;
	}

	li->next = NULL;
	li->data = data;
	current = li;
	last = li;
	++nElements;

        return data;
}

void * LinkedList::add( void * data, int sizeOfData )
{
	LISTITEM * li = (LISTITEM*)malloc( sizeof( LISTITEM ) );
	void * dataCopy;

	if( li == NULL ) return NULL;
        
	if( ( dataCopy = calloc( sizeOfData + 1, sizeof(char)) ) == NULL ) return NULL;  // sometimes strings are added, reserve space for terminating 0
	
	memcpy( dataCopy, data, sizeOfData );

	li->dataBufferAllocated = 1;
	if( last != NULL ) 
	{
		li->previous = last;
		last->next =  li;		
	}
	else
	{
		first = li;
		li->previous = NULL;
	}

	li->next = NULL;
	li->data = dataCopy;
	current = li;
	last = li;
	++nElements;
        return dataCopy;
}


void LinkedList::removeCurrent()
{
	if( current == NULL ) return;

	if( current->previous != NULL ) current->previous->next = current->next;
	else first = current->next;

	if( current->next != NULL ) current->next->previous = current->previous;
	else last = current->previous;

	if( current->dataBufferAllocated ) free( current->data );

	free( current );
	--nElements;
}

int LinkedList::goToPrevious()
{
  if( current == first ) return 0;
   current = current->previous;
   return 1;
}

int LinkedList::goToNext()
{
  if( current == last ) return 0;
   current = current->next;
   return 1;
}

void * LinkedList::getCurrent()
{
  if( current!= NULL ) return current->data;

  return NULL;
}

void * LinkedList::getCurrentGoToNext()
{
  void * data;
  if( current!= NULL )
    {
      data = current->data;
      if( current->next ) current = current->next;
      return data;
    }
  
  return NULL;
}

void * LinkedList::getCurrentGoToPrevious()
{
  void * data;
  
  if( current != NULL )
    {
      data = current->data;
      if( current->previous ) current = current->previous;
      return data;
    }
  
  return NULL;
}

