
#include "motorolaSrecordReader.h"



MotorolaSrecordReader::MotorolaSrecordReader( char * name )
{
    output = NULL;
    outputSize = maxOutputSize = -1;
    dataByteSum = 0;
    setFileName(name);
}

int MotorolaSrecordReader::convert( char * buffer, int maxSize  )
{
   ifstream inf( fileName, ifstream::in );
   if (!inf.is_open()) return CANT_OPEN;

   output = buffer;
   maxOutputSize = maxSize;
   outputSize = 0;
   dataByteSum = 0;
   char buff[MAX_CHARS];
   int res;
   while( !inf.eof() )
   {
      inf.getline( buff, MAX_CHARS );
      res = processLine( buff ) ;
      if( res ) return res;
   }

   return OK;
}

int MotorolaSrecordReader::processLine( char * line )
{
    int len = strlen(line);
    if( len == 0 ) return OK;
    if( len < 2 )  return LINE_TOO_SHORT;
    if( len % 2 )  return LINE_ODD_CHARACTERS;
    if( line[0] != 'S' && line[0] != 's' ) return NO_S_RECORD_INDICATOR;
    switch( line[1] )
    {
        case '0': return OK;  // S0 does not contain any data
        break;
        case '1': return S1record( line + 2, len - 2 );
        break;
        case '2': return S2record( line + 2, len - 2 );
        break;
        case '3': return S3record( line + 2, len - 2 );
        break;
        case '5':
        case '7':
        case '8':
        case '9': return OK;  // S5,S7,S8,s9 do not contain any data
        break;
        default: return UNKNOWN_S_RECORD_TYPE;
        break;
    }
}

int MotorolaSrecordReader::S1record( char * buff, int len )
{
    if( len < 6 ) return LINE_TOO_SHORT;
    int count = getValueFromHexPair(buff[0],buff[1]);
    if( count > (len - 2) >> 1 ) return LINE_TOO_SHORT;
    if( checkBaseAddress(buff+2) ) return WRONG_BASE_ADDRESS;
    int crc = count + getValueFromHexPair(buff[2],buff[3]) + getValueFromHexPair(buff[5],buff[6]);
    return processRecord( crc, buff + 6 , len - 6 );
}

int MotorolaSrecordReader::S2record( char * buff, int len )
{
    if( len < 8 ) return LINE_TOO_SHORT;
    int count = getValueFromHexPair(buff[0],buff[1]);
    if( count > (len - 2) >> 1 ) return LINE_TOO_SHORT;
    if( checkBaseAddress(buff+2) ) return WRONG_BASE_ADDRESS;
    int crc = count +
              getValueFromHexPair(buff[2],buff[3])+
              getValueFromHexPair(buff[4],buff[5])+
              getValueFromHexPair(buff[6],buff[7]);
    return processRecord( crc, buff + 8 , len - 8 );
}

int MotorolaSrecordReader::S3record( char * buff, int len )
{
    if( len < 10 ) return LINE_TOO_SHORT;
    int count = getValueFromHexPair(buff[0],buff[1]);
    if( count > (len - 2) >> 1 ) return LINE_TOO_SHORT;
    if( checkBaseAddress(buff+2) ) return WRONG_BASE_ADDRESS;
    int crc = count +
              getValueFromHexPair(buff[2],buff[3])+
              getValueFromHexPair(buff[4],buff[5])+
              getValueFromHexPair(buff[6],buff[7])+
              getValueFromHexPair(buff[8],buff[9]);
    return processRecord( crc, buff + 10 , len - 10 );
}

int MotorolaSrecordReader::processRecord( int crc,  char * buffer, int len )
{
    if( len < 2 ) return LINE_TOO_SHORT;

    char d = 0;
    for( int i = 0; i < len-2; i+=2 )
    {
        if( maxOutputSize <= outputSize ) return NO_ROOM;
        d = (char)getValueFromHexPair( buffer[i], buffer[i+1] );
        *(output++) = d;
        ++outputSize;
        crc += d;
        dataByteSum += d;
    }
    int expectedCrc = getValueFromHexPair(buffer[len-2], buffer[len-1]);
    return ((expectedCrc+crc) & 0xFF) != 0xFF ? CRC_ERROR : OK;
}

unsigned int MotorolaSrecordReader::getValueFromHexPair( unsigned char c, unsigned char d)
{
    return (getValueFromHexChar(c)<< 4 ) + getValueFromHexChar(d);
}

unsigned int MotorolaSrecordReader::getValueFromHexChar( unsigned char c )
{
    c = toupper(c);
    if( c > '/' && c < ':' ) return (int) (c - '0');
    if( c > '@' && c < 'G' ) return (int) (c - 'A' + 10);
    return 0;
}


int MotorolaSrecordReader::checkBaseAddress(char * buff)
{
  unsigned int baseAddress;

  baseAddress  = ( getValueFromHexPair(buff[0],buff[1]) << 4 ) | getValueFromHexChar(buff[2]);
  return (baseAddress - BASE_ADDRESS);
}
