//:nihash.mth - modified from =================================
// chash.mth: Coal-hashing table class template methods.
// Copyright(c) 1994 Azarona Software. All rights reserved.
//------------------------------------------------------------
// 27.03.96 fix bug (cursor init to 0 wrong, accidently works in dos
// 05.02.97 Move into shash.h - works only if shash.h is only included 
// in a single object file, which is now the case for device stuff
//------------------------------------------------------------
// 27.06.07 REMOVE TEMPLATING, specialize to simplified NameInd type

#include <new>      // was new.h ??
#include "nihash.h"

//--------------- Code for NameInd ----------------------------------
unsigned long NameInd::Hash(const char *quay) { //elf hash from Flamig
  unsigned long h = 0;
  unsigned char *q = (unsigned char*) quay;
  while (*q) { // Black magic!
    h = (h << 4) + *(q)++ ;  // preserves source parenthesis, check result
    unsigned long g = h & 0xF0000000L;
    if (g) h ^= g >> 24;  h &= ~g;
  } return h;
}

//------------- Code for Hash table -----------------------------------
CoalHashTableNI::CoalHashTableNI(unsigned short tsize) {
// Sets up a hash table with tsize buckets. The data portion
// of the buckets are left unconstructed until used.
// Increase array size to avoid Flamig pointer decrement.
  pm = new hashparams(); // add s to hashparam !! 12.12.97
  pm->ok=0;
  buckets = (NameInd *) new char[(tsize+2) * sizeof(NameInd)];
  next = new unsigned short[tsize+2];
  inuse = new char[tsize+2];
  if (buckets && next && inuse) {
     pm->ok = 1;
     pm->num_buckets = tsize;
     for (unsigned i = 1; i <= pm->num_buckets; i++) inuse[i] = 0;
     Clear();
  }
  else {
     delete pm;
     delete[] inuse;  delete[] next;
     delete[] (char *)buckets; // As allocated, so deallocated
     printf("hash table create fails\n");
     exit(-1);
  }
}

CoalHashTableNI::~CoalHashTableNI() {
  if (pm->ok) {
     Clear();
     // Must adjust pointers back to real start of arrays
     printf("Hash Table Destructor Called\n");
     delete pm;
     delete[] inuse;  delete[] next;
     delete[] (char *)(buckets); // As allocated, so deallocated
  }
}

void CoalHashTableNI::Clear() {
// Clears the table, destroying the data for all buckets in use.
  unsigned i;
//  for (i = 1; i<=pm->num_buckets; i++) {
//      if (inuse[i]) (buckets+i)->TYPE::~TYPE();
//  }
  for (i = 1; i<=pm->num_buckets; i++) inuse[i] = 0;
  for (i = 1; i<=pm->num_buckets; i++) next[i] = 0;
  pm->cursor = 1;  // ??? 27.03.96
  pm->num_entries = 0; pm->num_collisions = 0; pm->num_comparisons = 0;
}

int CoalHashTableNI::Insert(const NameInd &x) {
// Inserts a new entry having the specified key. If a matching
// entry is found, DUPLICATE_KEY is returned, else SUCCESS or
// TABLE_FULL is returned.
  int rv;
  long old_num_compares = pm->num_comparisons;
  unsigned h = unsigned(NameInd::Hash(x.Key()) % (pm->num_buckets)) + 1;
  while (inuse[h]) { // Okay, we've got collision
     (pm->num_comparisons)++;
     if (buckets[h].Match(x.Key())) {
       if (buckets[h].Index()==x.Index()) rv = DUPL_ENTRY;
       else rv = DUPL_KEY;
       goto we_done;
     }
     unsigned p = h;
     h = next[h];
     if (h == 0) { // End of list, search for place to add node
        while(1) {
          if (pm->cursor == pm->num_buckets) {
             rv = TABLE_FULL; goto we_done;
          }
          if (inuse[pm->cursor] == 0) break;
          ++(pm->cursor);
        }
        h = pm->cursor;          // This is the new node for list
        next[p] = (pm->cursor)++;
        break;
     }
  }
  new(buckets+h) NameInd(x); // In-place copy construct the data
  inuse[h] = 1;
  next[h] = 0;
  (pm->num_entries)++;
  rv = SUCCESS;
we_done:
  if (pm->num_comparisons != old_num_compares) (pm->num_collisions)++;
  return rv;
}

NameInd *CoalHashTableNI::Search(const char* key)  {
// Searches for a bucket having specified key. Returns pointer
// to the data in the bucket if found, else returns 0.
  unsigned h = unsigned(NameInd::Hash(key) % (pm->num_buckets)) + 1;
  while(1) {
    if (inuse[h] == 0) break;
    if (buckets[h].Match(key)) return buckets + h;
    h = next[h];
    if (h == 0) break;
  }
  return 0; // not found
}
int CoalHashTableNI::GetIndex(const char* key)  {
// Searches for a bucket having specified key. Returns pointer
// to the data in the bucket if found, else returns 0.
  unsigned h = unsigned(NameInd::Hash(key) % (pm->num_buckets)) + 1;
  while(1) {
    if (inuse[h] == 0) break;
    if (buckets[h].Match(key)) return (buckets + h)->Index();
    h = next[h];
    if (h == 0) break;
  }
  return -1; // not found
}

int CoalHashTableNI::Exists(const char* key) {
// Searches for a bucket having specified key.  If found
// returns 1, else 0.  Copy code from Search
  unsigned h = unsigned(NameInd::Hash(key) % (pm->num_buckets)) + 1;
  while(1) {
    if (inuse[h] == 0) break;
    if (buckets[h].Match(key)) return KEY_EXISTS;
    h = next[h];
    if (h == 0) break;
  }
  return 0; // not found
}

int CoalHashTableNI::Update(const NameInd &x) {
// Updates the bucket containing x, if found, with new data.
// Returns SUCCESS or NOT_FOUND.
  NameInd *p = Search(x.Key());
  if (p) {
     *p = x;
     return SUCCESS;
  }
  return NOT_FOUND;
}

