//:chash.h - version for shared memory (28.03.96), from ===
// chash.h: Coalesced hashing table class template.
// Copyright(c) 1994 Azarona Software. All rights reserved.
//---------------------------------------------------------
// 02.14.96 - limit key to char*
//==========================================================
// 27.06.07  Version with TEMPLATES removed !!
#ifndef NIHASH_H__
#define NIHASH_H__

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

//#include <iostream>
//using namespace std;

//-----------------------------------------------------------------
const int SUCCESS         =  0;
const int ALLOC_FAILURE   =  1;
const int NOT_FOUND       =  2;
const int DUPL_KEY        =  3;  // same key, different index, TILT!
const int TABLE_FULL      =  4;
const int NOT_SUPPORTED   =  5;
const int DUPL_ENTRY      =  6;  // key AND index same, so ok?
const int NOT_OWNER       =  7;  // for shmem version
const int KEY_EXISTS      =  8;  // add 27.06.07  
//
//------ Minimal 'Entry' class for hash ----------------------------
class NameInd {
  int index;
  char key[33]; // ensures null-term?
 public:
  NameInd() { index=0; key[0] = '\0';  }
  NameInd(const char* c, int i);
  ~NameInd() {}
  void set(const char* c,int i);
  int Match(const char* quay) const { return strncmp(key,quay,32) == 0;  }
  const int Index() const { return index; }
  const char* Key() const { return key; }
  static unsigned long Hash(const char *quay);
};

inline NameInd::NameInd(const char*c, int i) {
  if (strlen(c)<=32) { index=i; strcpy(key,c); }
  else { printf("Name too long: %s\n",c); exit(1); }
}
inline void NameInd::set(const char* c,int i) {
  if (strlen(c)<=32) { index=i; strcpy(key,c); }
  else { printf("Name too long: %s\n",c); exit(1); }
}
// ---------- Hash Class for NameInd -----------------------------
struct hashparams { // defined for shared mem version
unsigned short num_buckets, ok, cursor;
unsigned short num_entries, num_collisions, num_comparisons;
};

class CoalHashTableNI {
// Hashes entries of type TYPE. It's assumed that TYPE knows how
// to match for KEY's. (TYPE should probably contain a KEY.)
protected:
  hashparams *pm; 
  NameInd *buckets;  // Data for the buckets
  unsigned short *next; // Next pointers for the buckets
  char *inuse;    // Inuse flags for the buckets
public:
  CoalHashTableNI(unsigned short tsize);   // for normal allocation
  virtual ~CoalHashTableNI();  // what to do ??
  void Clear();
  int Insert(const NameInd &x);
  int Update(const NameInd &x);
  NameInd *Search(const char* key); // return struct
  int GetIndex(const char* key);    // just return index or -1
  int Exists(const char* key);      // 0 = not found, else 1
  int Delete(const char* key);
  unsigned NumBuckets() const;
  long NumEntries() const;
  long NumCollisions() const;
  long NumComparisons() const;
  int OK() const;
};

inline unsigned CoalHashTableNI::NumBuckets() const {
  return pm->num_buckets;
}
inline long CoalHashTableNI::NumEntries() const {
  return pm->num_entries;
}
inline long CoalHashTableNI::NumCollisions() const {
  return pm->num_collisions;
}
inline long CoalHashTableNI::NumComparisons() const {
  return pm->num_comparisons;
}
inline int CoalHashTableNI::OK() const {
  return pm->ok;
}
inline int CoalHashTableNI::Delete(const char* key) {
// Deletion not supported for closed hashing
// If deletion desired, go to doubly linked list
  return NOT_SUPPORTED;
}
#endif
