
#include "objectDictionary.h"


typedef struct
{
  int code;
  char name[16];
}TYPE_LIST;

TYPE_LIST ObjectTypeList[] =
  {
    { 0x07, "VAR" },
    { 0x08, "ARRAY" },
    { 0x09, "RECORD" }
  };
  
TYPE_LIST VariableTypeList[] =
  {
    { CO_BOOL1, "BOOL1"   },
    { CO_INT8, "INT8"    },
    { CO_INT16, "INT16"   },
    { CO_INT32, "INT32"   },
    { CO_UINT8, "UINT8"   },
    { CO_UINT16, "UINT16"  },
    { CO_UINT32, "UINT32"  },
    { CO_REAL32, "REAL32"  },
    { CO_VIS_STR, "VIS_STR" },
    { CO_OCTET_STR, "OCTET_STR" },
    { CO_UNICODE_STR, "UNICODE_STR" },
    { CO_TIME_OF_DAY, "TIME_OF_DAY" },
    { CO_TIME_DIFF, "TIME_DIFF" },
    { CO_BIT_STR, "BIT_STR" },
    { CO_DOMAIN, "DOMAIN" },
    { CO_INT24, "INT24" },
    { CO_REAL64, "REAL64" },
    { CO_INT40, "INT40" },
    { CO_INT48, "INT48" },
    { CO_INT56, "INT56" },
    { CO_INT64, "INT64" },
    { CO_UINT24, "UINT24" },
    // 0x17 - reserved
    { CO_UINT40, "UINT40" },
    { CO_UINT48, "UINT48" },
    { CO_UINT56, "UINT56" },
    { CO_UINT64, "UINT64" }
  };

TYPE_LIST AccessTypeList[] =
  {  // warning: codes for access are not defined by the standard, were introduced by K.B.
    { 0x01, "RO" },
    { 0x02, "WO" },   
    { 0x03, "RW" },
    { 0x04, "RWR" },   // read/wite on process input
    { 0x05, "RWW" },   // read/write on process output
    { 0x11, "CONST" }
  };

ODentry::ODentry()
{
  ODentry(0,-1);
}

ODentry::ODentry( int idx, int subidx )
{
  index = idx;
  subindex = subidx; 
  objectTypeCode = VAR;   // assume VAR as a default object type
  dataTypeCode = CO_INT16;     // assume INT16 as a default data type
  accessTypeCode = 0x01;   // assume RO as a defaulty access mode
}

ODentry::~ODentry()
{
}

int ODentry:: getDataTypeSize( int dType )
{
  switch( dType )
    {
    case CO_NO_DATA: return 0;
      break;
    case CO_VIS_STR:
    case CO_OCTET_STR: 
    case CO_BIT_STR:   
    case CO_UNICODE_STR: 
    case CO_DOMAIN:
      Lg << ERROR << "--Error (internal): getDataSize() called for non-prmitive data type" << endl;
      return 8;
      break;
    case CO_BOOL1: return 1;
      break;
    case CO_INT8: return 1;
      break;
    case CO_INT16: return 2;
      break;
    case CO_INT32: return 4;
      break;
    case CO_UINT8: return 1;
      break;
    case CO_UINT16: return 2;
      break;
    case CO_UINT32: return 4;
      break;
    case CO_REAL32: return 4;
      break;
    case CO_INT24: return 3;
      break;
    case CO_REAL64: return 8;
      break;
    case CO_INT40: return 5;
      break;
    case CO_INT48: return 6;
      break;
    case CO_INT56: return 7;
      break;
    case CO_INT64: return 8;
      break;
    case CO_UINT24: return 3 ;
      break;  
    case CO_UINT40: return 5;
      break;
    case CO_UINT48:  return 6;
      break;
    case CO_UINT56:  return 7;
      break;
    case CO_UINT64:  return 8;
      break;
    default:
      Lg << ERROR << "--Error (internal): getDataSize() called for unknown data type" << endl; 
      return 8;
      break;     
    }
  return 0;
}

void ODentry::assign( ODvariable * var, char * data, int size )
{
  switch( dataTypeCode )
    {
    case CO_NO_DATA:
      break;
    case CO_VIS_STR:
    case CO_OCTET_STR: 
    case CO_BIT_STR:   
    case CO_UNICODE_STR: 
    case CO_DOMAIN:
      var->allocate( size + 1 ); // allocate() frees a buffer, if any was previuosly allocated for the variable
      memcpy( var->v.valDOMAIN, data, size );
      break;
    default:
      memcpy( &(var->v), data, size );
      break;     
    }
  var->set();
}

void ODentry::store( ODvariable * var, char *keyValue )  //keyValue is a string representing data, like "1.045"
{ 
  // check for $NODEID macro, the only one allowed
  char * pos = strcasestr( keyValue, "$NODEID" );
  if( pos != NULL )
    {
      var->useNodeIDasOffset(1);
      keyValue = strstr( pos, "+" );
      if( keyValue == NULL ) keyValue = "0";
      else ++keyValue;  // skip the "+" character
      //cout << "MACRO $NODEID found, keyValue = " << keyValue << endl;
    }

  
  switch( dataTypeCode )
  {
    case CO_NO_DATA:
      break;
    case CO_BOOL1:
      var->v.valBOOL1 = ( strtol( keyValue, NULL, 0 ) != 0 ); var->set();
      break;
      
    case CO_INT8:
      var->v.valINT8 = (char) strtol(keyValue, NULL, 0); var->set();
      break;

    case CO_INT16:
      var->v.valINT16 = (short) strtol(keyValue, NULL, 0); var->set();
      break;

    case CO_INT32:
      var->v.valINT32 =  strtol(keyValue, NULL, 0); var->set();
      break;
 
    case CO_UINT8:
      var->v.valUINT8 = (unsigned char)strtoul(keyValue, NULL, 0); var->set();
      break;

    case CO_UINT16:
      var->v.valUINT16 = (unsigned short)strtoul(keyValue, NULL, 0); var->set();
      break;

    case CO_UINT32:
      var->v.valUINT32 = strtoul(keyValue, NULL, 0); var->set();
      break;

    case CO_REAL32:
      var->v.valREAL32 = strtof(keyValue, NULL); var->set();
      break;

    case CO_VIS_STR:
    case CO_OCTET_STR: 
    case CO_BIT_STR:
      var->allocate( strlen(keyValue) + 1 );   // allocate() frees a buffer, if any was previuosly allocated for the variable
      strcpy( var->v.valVIS_STR, keyValue );
      var->set();
      break;

    case CO_UNICODE_STR:
      Lg << ERROR << "--Error: ObjectDictionary::store(): UNICODE_STR not handled yet." << endl;
      break;

    case CO_TIME_OF_DAY:
      Lg << ERROR << "--Error: ObjectDictionary::store():  TIME_OF_DAY not handled yet." << endl;
      break;

    case CO_TIME_DIFF:
      Lg << ERROR << "--Error: ObjectDictionary::store():  TIME_DIFF not handled yet." << endl;
      break;

    case CO_DOMAIN:
      Lg << ERROR << "--Error: ObjectDictionary::store():  DOMAIN not handled yet." << endl;
      break;

    case CO_INT24:
      var->v.valINT24 = strtol(keyValue, NULL, 0); var->set();
      break;

    case CO_REAL64:
      var->v.valREAL64 = strtod(keyValue, NULL ); var->set();
      break;

    case CO_INT40:
       var->v.valINT40 = strtoll(keyValue, NULL, 0); var->set();
      break;

    case CO_INT48:
       var->v.valINT48 = strtoll(keyValue, NULL, 0); var->set();
       break;

    case CO_INT56:
       var->v.valINT56 = strtoll(keyValue, NULL, 0); var->set();
       break;

    case CO_INT64:
       var->v.valINT64 = strtoll(keyValue, NULL, 0); var->set();
       break;

    case CO_UINT24: 
      var->v.valUINT24 = (unsigned int) strtoul(keyValue, NULL, 0); var->set();
       break;  

    case CO_UINT40: 
      var->v.valUINT40 = (unsigned long long) strtoull(keyValue, NULL, 0); var->set();
      break;

    case CO_UINT48:  
      var->v.valUINT48 = (unsigned long long) strtoull(keyValue, NULL, 0); var->set();
      break;

    case CO_UINT56:  
      var->v.valUINT56 = (unsigned long long) strtoull(keyValue, NULL, 0); var->set();
      break;

    case CO_UINT64:  
      var->v.valUINT64 = (unsigned long long) strtoull(keyValue, NULL, 0); var->set();
      break;

    default:
      Lg << WARNING <<  "-Warning: unexpected data type code: " << dataTypeCode << ", ( 0x" << hex << dataTypeCode << ")" << dec << endl;
      break;
  }

}

char *ODentry::restore( ODvariable * var, char *keyValue )
{
 
  switch(dataTypeCode)
    {
    case CO_VIS_STR:
    case CO_OCTET_STR: 
    case CO_BIT_STR:
      if( var->isBufferAllocated() )strcpy( keyValue, var->v.valVIS_STR );        
      else strcpy( keyValue, "-not set-" );
      break;

    case CO_UNICODE_STR:
      Lg << ERROR << "--Error: ObjectDictionary::store(): UNICODE_STR not handled yet." << endl;
      strcpy( keyValue, "-not handled yet-" );
      break;

    case CO_TIME_OF_DAY:
      Lg << ERROR << "--Error: ObjectDictionary::store():  TIME_OF_DAY not handled yet." << endl;
      strcpy( keyValue, "-not handled yet-" );
      break;

    case CO_TIME_DIFF:
      Lg << ERROR << "--Error: ObjectDictionary::store():  TIME_DIFF not handled yet." << endl;
      strcpy( keyValue, "-not handled yet-" );
      break;

    case CO_DOMAIN:
      Lg << ERROR << "--Error: ObjectDictionary::store():  DOMAIN not handled yet." << endl;
      strcpy( keyValue, "-not handled yet-" );
      break;
    default :
      if( !var->isSet() ) strcpy(keyValue, "-not set-" );
      else return ODvalueToString( var->v, dataTypeCode, keyValue );
      break;
    }
  return keyValue;
}

char * ODentry::ODvalueToString( OD_value v, int dataType, char *keyValue )
{
  switch( dataType )
  {
    case CO_NO_DATA:  
      strcpy( keyValue, "-not set-" );
      break;

    case CO_BOOL1:      
      if( v.valBOOL1 ) strcpy( keyValue, "1" );
      else strcpy( keyValue, "0" );        
      break;
      
    case CO_INT8:
      sprintf( keyValue, "%d", v.valINT8 ); 
      break;

    case CO_INT16:
      sprintf( keyValue, "%d", v.valINT16 ); 
      break;

    case CO_INT32:
      sprintf( keyValue, "%d", v.valINT32 );      
      break;
 
    case CO_UINT8:
      sprintf( keyValue, "%u", v.valUINT8 );     
      break;

    case CO_UINT16:
      sprintf( keyValue, "%u", v.valUINT16 );       
      break;

    case CO_UINT32:
      sprintf( keyValue, "%u", v.valUINT32 ); 
      break;

    case CO_REAL32:
      sprintf( keyValue, "%f", v.valREAL32 ); 
      break;
    case CO_INT24:
      sprintf( keyValue, "%d", v.valINT24 );  
      break;

    case CO_REAL64:
      sprintf( keyValue, "%g", v.valREAL64 );  
      break;

    case CO_INT40:
      sprintf( keyValue, "%ld", v.valINT40 );   
      break;

    case CO_INT48:
      sprintf( keyValue, "%ld", v.valINT48 );
      break;

    case CO_INT56:
      sprintf( keyValue, "%ld", v.valINT56 );     
      break;

    case CO_INT64:
      sprintf( keyValue, "%ld", v.valINT64 );      
      break;

    case CO_UINT24: 
      sprintf( keyValue, "%u", v.valUINT24);      
      break;  

    case CO_UINT40: 
      sprintf( keyValue, "%lu", v.valUINT40);
      break; 
      
    case CO_UINT48:  
      sprintf( keyValue, "%lu", v.valUINT48);       
      break;

    case CO_UINT56:  
      sprintf( keyValue, "%lu", v.valUINT56);      
      break;

    case CO_UINT64:  
      sprintf( keyValue, "%lu", v.valUINT64);      
      break;

    default:
      Lg << WARNING << "-Warning: unexpected data type code: " << dataType << ", ( 0x" << hex << dataType << ")" << dec << endl;
      break;
  }
  return keyValue;
}

ObjectDictionary::ObjectDictionary()
{
  entriesList = new LinkedList();

  for( int i = 0; i < HASH_TABLE_SIZE; i++ ) hashTable[i] = new LinkedList();
}

ObjectDictionary::~ObjectDictionary()
{ 
  for( int i = 0; i < HASH_TABLE_SIZE; i++ ) delete hashTable[i];
  delete entriesList;
}

ODentry * ObjectDictionary::add( ODentry * entry )
{  
  ODentry * d;
  int sidx = entry->subindex;
  //if( (sidx = entry->subindex) < 0 ) sidx = 0;
   
  d = (ODentry *)entriesList->add( entry, sizeof( ODentry ));
  hashTable[ elfHash( entry->index, sidx )]->add( entriesList->getCurrentListItem() );  // store the location of the Linked List Item
  
  return d; 
}

unsigned int ObjectDictionary::elfHash( unsigned short idx, unsigned char sidx )
{
  unsigned long b  = ((idx << 8) | sidx);
  char *ptr = (char *)&b;
  unsigned long g, h = 0;
  
  for( unsigned int i = 0; i < sizeof(long); i++ )
    {
      h = ( h << 4 ) + *ptr++;
      g = h & 0xF0000000;
      if( g )  h ^= g >> 24;
      h &= ~g;
    }
  return (unsigned int)(h % HASH_TABLE_SIZE);
}

ODentry * ObjectDictionary::seek( int index, int subindex ) 
{

  //if( subindex < 0 ) subindex = 0;

  LISTITEM * li;
  LinkedList *ll = hashTable[elfHash(index,subindex)];
  ODentry * e;
  ll->goToFirst();
  if(ll->getCurrent() == NULL ) return NULL; // the list is empty, no such ODentry(index, subindex)

  do 
  {
    li = (LISTITEM *) ll->getCurrent();   
    e = (ODentry*)li->data;
    if( e->index == index )
      { 
        if( (e->subindex == subindex) /* || (e->subindex == -1 && subindex < 1 ) */ )
          {
            entriesList->move( li );
            return e;
          }
      }
  } while( ll->goToNext() ); 

  return NULL;
}

ODentry * ObjectDictionary::linearSeek( int index, int subindex ) 
{
  ODentry * e;

  entriesList->goToFirst();
  if( entriesList->getCurrent() == NULL ) return NULL; // the list is empty

  do 
  {
    e = (ODentry *) entriesList->getCurrent();   
    if( e->index == index )
      { 
        if( e->subindex == subindex ) return e;
        // if( e->subindex == -1 && subindex < 1 ) return e;  // some entries do not have subindecies, accept 0 as a valid subindex in that case
      }
  } while( entriesList->goToNext() ); 

  return NULL;
}

ODentry * ObjectDictionary::getNext()
{
  if( entriesList->goToNext() ) return getCurrent();
  return NULL;
}

void ObjectDictionary::dump()
{
  char buff[256];
  ODentry * e;

  Lg << DBG << endl << "Dump of Object Dictionary" << endl;

  entriesList->goToFirst();
  do 
  {
    e = (ODentry *) entriesList->getCurrent();
    Lg << DBG << "----[0x" << hex << e->index;
    if( e->subindex > -1 ) Lg << DBG << ", " << e->subindex;
    Lg << DBG << "]-----------------------------" << dec << endl;
    Lg << DBG << " Parameter name: " << e->parameterName << endl;
    Lg << DBG << " subNumber: " << e->subNumber << endl;
    Lg << DBG << " object type: " << getObjectTypeString( e->objectTypeCode ) << endl;
    Lg << DBG << " data type: " << getVariableTypeString( e->dataTypeCode ) << endl;
    Lg << DBG << " access type: " << getAccessTypeString( e->accessTypeCode ) << endl;
    Lg << DBG << " PDO mappable: " ;
    if( e->PDOmappable  ) Lg << DBG << "YES" << endl; else Lg << DBG << "NO" << endl;
    Lg << DBG << " default value: " << e->restoreDefaultValue(buff) << endl;    
    Lg << DBG << " current value: " << e->restoreCurrentValue(buff) << endl;
    Lg << DBG << " high limit: " << e->restoreHighLimit(buff) << endl;
    Lg << DBG << " low limit: " << e->restoreLowLimit(buff) << endl;
    
  } while( entriesList->goToNext() );

  Lg << DBG << endl;
}


int ObjectDictionary::getObjectTypeCode( char * objectTypeString )
{
  unsigned int i;

  objectTypeString = toUpperString( objectTypeString );

  for( i = 0; i < sizeof(ObjectTypeList)/sizeof(TYPE_LIST); i++ )
    if( ! strcmp( objectTypeString , ObjectTypeList[i].name ) ) return ObjectTypeList[i].code;
    
  return 0;
}

int ObjectDictionary::getDataTypeCode( char * varTypeString )
{
  unsigned int i;

  varTypeString = toUpperString( varTypeString );

  for( i = 0; i < sizeof(VariableTypeList)/sizeof(TYPE_LIST); i++ )
    if( ! strcmp( varTypeString , VariableTypeList[i].name ) ) return VariableTypeList[i].code;
    
  return 0;
}

int ObjectDictionary::getAccessTypeCode( char * accessTypeString )
{
  unsigned int i;

  // if( !strlen(accessTypeString) ) return 6;  // return "CONST", if no acccess mode defined
  accessTypeString = toUpperString( accessTypeString );
  for( i = 0; i < sizeof(AccessTypeList)/sizeof(TYPE_LIST); i++ )
    if( ! strcmp( accessTypeString , AccessTypeList[i].name ) ) return AccessTypeList[i].code;
 
  return 0;
}

char * ObjectDictionary::getObjectTypeString( int objectTypeCode )
{
  unsigned int i;

  for( i = 0; i < sizeof(ObjectTypeList)/sizeof(TYPE_LIST); i++ )
    if( objectTypeCode == ObjectTypeList[i].code )  return ObjectTypeList[i].name;
    
  return "";
}

char * ObjectDictionary::getVariableTypeString( int variableTypeCode )
{
  unsigned int i;

  for( i = 0; i < sizeof(VariableTypeList)/sizeof(TYPE_LIST); i++ )
    if( variableTypeCode == VariableTypeList[i].code ) return VariableTypeList[i].name;
    
  return "";
}

char * ObjectDictionary::getAccessTypeString( int accessTypeCode )
{
  unsigned int i;

  for( i = 0; i < sizeof(AccessTypeList)/sizeof(TYPE_LIST); i++ )
    if( accessTypeCode == AccessTypeList[i].code ) return AccessTypeList[i].name;
    
  return "";
}

