#ifndef __OBJECT_DICTIONARY_H__
#define __OBJECT_DICTIONARY_H__


#include "loggerClass/log.h"
#include "linkedList.h"
#include "canDataTypes.h"
#include "edsParser.h"
     
extern Log Lg;


typedef struct
{
  int index;
  int subindex;
} OD_ENTRY_IDX;

class ODvariable
{
  int is_set;  
  int use_nodeID_as_offset;
  int allocatedBufferLength;
 public: 
  OD_value v;
  
  inline ODvariable() { use_nodeID_as_offset = is_set = allocatedBufferLength = 0; }
  inline ~ODvariable() { if(allocatedBufferLength) free(v.valDOMAIN); allocatedBufferLength = 0; }
  inline void allocate( int l ) { if( allocatedBufferLength ) free( v.valDOMAIN ); v.valDOMAIN = calloc( l, sizeof(char) ); allocatedBufferLength = l; }
  inline void freeBuffer() { if( allocatedBufferLength ) free( v.valDOMAIN ); allocatedBufferLength = 0; }
  inline int  isBufferAllocated() { return allocatedBufferLength; }  
  inline int  getAllocatedBufferLength() { return allocatedBufferLength; }
  inline void set() { is_set = 1; }
  inline int  isSet() { return is_set; }    // tells, if the variable was set during the EDS/DCF parsing process
  inline void useNodeIDasOffset( int use ) { use_nodeID_as_offset = ( use != 0 ); }
  inline int  useNodeIDoffsetUsed() { return use_nodeID_as_offset; }
};

class ODentryText
{
 public:
  inline ODentryText() 
    { 
      index = -1; 
      subindex = -1;  
      parameterName[0] = '\0';
      subNumber[0] = '\0';
      objectTypeCode[0] = '\0';
      dataTypeCode[0] = '\0';
      accessTypeCode[0] = '\0'; 
      defaultValue[0] = '\0';
      highLimit[0] = '\0';
      dcfParameterValue[0] = '\0';
      lowLimit[0] = '\0';
      compactSubObj[0] = '\0';
      PDOmappable[0] = '\0'; 
    }

  int index;
  int  subindex;
  char parameterName[PARAMETER_NAME_LEN + 1];
  char subNumber[MAX_LINE_LENGTH];
  char objectTypeCode[MAX_LINE_LENGTH];
  char dataTypeCode[MAX_LINE_LENGTH];
  char accessTypeCode[MAX_LINE_LENGTH]; 
  char defaultValue[MAX_LINE_LENGTH];
  char highLimit[MAX_LINE_LENGTH];
  char dcfParameterValue[MAX_LINE_LENGTH];
  char lowLimit[MAX_LINE_LENGTH];
  char compactSubObj[MAX_LINE_LENGTH];
  char PDOmappable[MAX_LINE_LENGTH];
  int  fileLine_start;
  int  fileLine_end;
};

class ODentry
{
 private:
  void store( ODvariable * v, char *keyValue );
  char * restore( ODvariable * v, char *keyValue );
 
 public:

  ODentry();
  ODentry( int index, int subindex );
  ~ODentry();
  char parameterName[PARAMETER_NAME_LEN + 1];
  int index;
  int subindex;
  int subNumber;
  int objectTypeCode;
  int dataTypeCode;
  int accessTypeCode;  
  ODvariable defaultValue;
  ODvariable currentValue;
  ODvariable highLimit;
  ODvariable lowLimit;
  ODvariable dcfParameterValue;
  int CompactSubObj;
  int PDOmappable;

  static int getDataTypeSize( int dType );
  void assign( ODvariable * var, char * data, int size ); 
  inline void assignDefaultValue( char * data ) { assign( &defaultValue, data, getDataTypeSize(dataTypeCode)); }
  inline void assignCurrentValue( char * data ) { assign( &currentValue, data, getDataTypeSize(dataTypeCode)); }
  inline void assignHighLimit( char * data ) { assign( &highLimit, data, getDataTypeSize(dataTypeCode)); }
  inline void assignLowLimit( char * data ) { assign( &lowLimit, data, getDataTypeSize(dataTypeCode)); } 
  inline void assignDcfParameterValue( char * data )  { assign( &dcfParameterValue, data, getDataTypeSize(dataTypeCode)); } 
  inline void assignDefaultValue( char * data, int size ) { assign( &defaultValue, data, size); }
  inline void assignCurrentValue( char * data, int size ) { assign( &currentValue, data, size); }
  inline void assignHighLimit( char * data, int size ) { assign( &highLimit, data, size); }
  inline void assignLowLimit( char * data, int size ) { assign( &lowLimit, data, size); } 
  inline void assignDcfParameterValue( char * data, int size )  { assign( &dcfParameterValue, data, size); } 


  inline void storeDefaultValue( char *keyValue ) { store( &defaultValue, keyValue ); }  
  inline void storeDcfParameterValue( char *keyValue ) { store( &dcfParameterValue, keyValue ); }  
  inline void storeCurrentValue( char *keyValue ) { store( &currentValue, keyValue ); }
  inline void storeHighLimit( char *keyValue )    { store( &highLimit, keyValue ); }
  inline void storeLowLimit( char *keyValue )     { store( &lowLimit, keyValue ); }

  inline char * restoreDefaultValue( char *keyValue ) { return restore( &defaultValue, keyValue ); } 
  inline char * restoreCurrentValue( char *keyValue ) { return restore( &currentValue, keyValue ); }
  inline char * restoreHighLimit( char *keyValue )    { return restore( &highLimit, keyValue ); }
  inline char * restoreLowLimit( char *keyValue )     { return restore( &lowLimit, keyValue ); }
  inline OD_value * getDefaultValuePtr() { return &(defaultValue.v); }
  inline OD_value * getCurrentValuePtr() { return &(currentValue.v); }
  inline void copyDefaultToCurrentValue() 
    {
      memcpy( &currentValue, &defaultValue, sizeof( ODvariable ));
      if( defaultValue.getAllocatedBufferLength() ) 
        {
          currentValue.allocate(defaultValue.getAllocatedBufferLength());
          memcpy( &(currentValue.v), &(defaultValue.v),defaultValue.getAllocatedBufferLength()); 
        }
    }
  inline char * getParameterName() { return parameterName; }
  char *ODvalueToString( OD_value v, int dataType, char *keyValue );
};

const int HASH_TABLE_SIZE = 211;   // should be a prime number
class ObjectDictionary
{

 private:
  LinkedList * entriesList;
  LinkedList * hashTable[HASH_TABLE_SIZE];  
  unsigned int elfHash( unsigned short idx, unsigned char sidx );

 public: 
  
  ObjectDictionary();
  ~ObjectDictionary();

  static int getObjectTypeCode( char * objectTypeString );
  static int getDataTypeCode( char * varTypeString );
  static int getAccessTypeCode( char * accessTypeString );
  static char * getObjectTypeString( int objectTypeCode );
  static char * getVariableTypeString( int variableTypeCode );
  static char * getAccessTypeString( int accessTypeCode );
  static inline char * toUpperString( char * s ) { char * b = s; while(*s) { *s = toupper(*s);  ++s; } return b;}
  ODentry * linearSeek( int index, int subindex ); 
  ODentry * seek( int index, int subindex );
  inline ODentry * seek(  OD_ENTRY_IDX i ) { return seek( i.index, i.subindex ); }
  inline ODentry *getCurrent() {return (ODentry*)entriesList->getCurrent(); }
  ODentry *getNext();
  inline void goTop() {  entriesList->goToFirst(); }
  ODentry * add( ODentry * entry );
  void dump();
};

#endif
