
#include "pdo.h"
#include "canNodeImage.h"

int getMask( int maskLen )
{
  if( maskLen > 31 ) return 0xFFFFFFFF;
  return (1 << maskLen) - 1;
}

PDOclient::PDOclient( CanNodeImage * parent )
{
  owner = parent;
  nodeID = owner->getNodeId();
  for( int i = 0; i < MAX_PDO; i++ ) 
  {
      rPdoId[i] = 0x0;  //0x200 + i * 0x100 + nodeID;
      tPdoId[i] = 0x0;  //0x180 + i * 0x100 + nodeID;
  }
  setName( "PDOclient" );
  reset();
}

int PDOclient::add( PdoMapping * pdoMapping, int pdoNo, int itemNo, int mapping )
{
    pdoMapping->pdoVariable[pdoMapping->pdoVariablesCount].index = ( mapping & 0xFFFF0000 ) >> 16;
    if(pdoMapping->pdoVariable[pdoMapping->pdoVariablesCount].index == 0 ) return 0; // sometimes empty mapping is reported. Ignore
    pdoMapping->pdoVariable[pdoMapping->pdoVariablesCount].subindex = (mapping & 0x0000FF00) >> 8;
    pdoMapping->mask[pdoNo][itemNo].shift = ( itemNo == 0 ? 0 :  pdoMapping->mask[pdoNo][itemNo-1].nBytes + pdoMapping->mask[pdoNo][itemNo-1].shift );
    pdoMapping->mask[pdoNo][itemNo].maskPattern = getMask(mapping & 0x000000FF);
    pdoMapping->mask[pdoNo][itemNo].nBytes = (mapping & 0xFF) >> 3;
    pdoMapping->pdoVariable[pdoMapping->pdoVariablesCount].dataSize = pdoMapping->mask[pdoNo][itemNo].nBytes;

    if((pdoMapping->pdoVariable[pdoMapping->pdoVariablesCount].dataPtr = 
        owner->getODcurrentValuePtr(pdoMapping->pdoVariable[pdoMapping->pdoVariablesCount].index, 
                                    pdoMapping->pdoVariable[pdoMapping->pdoVariablesCount].subindex)) == NULL )
      {    
       pdoMapping->mask[pdoNo][itemNo].maskPattern = 0;  // indicate no item found in OD, variable will be ignored during PDO handling
        Lg << WARNING << "-Warning: mapping PDO to non existing OD element ignored. Index = 0x"<< hex 
           <<pdoMapping->pdoVariable[pdoMapping->pdoVariablesCount].index << dec 
           <<  ", subindex = " <<pdoMapping->pdoVariable[pdoMapping->pdoVariablesCount].subindex << endl;
        return 0;
      }
    else  pdoMapping->mask[pdoNo][itemNo].mappedVariableIndex = (pdoMapping->pdoVariablesCount)++;            
    return 1;
}

void  PDOclient::addRpdoMapping( int pdoNr, int itemNr, int mapping )
{  
  if(add( &rPdoMapping, pdoNr, itemNr, mapping ) )
    {
      rPdoMapping.pdoVariable[rPdoMapping.pdoVariablesCount-1].pdos[rPdoMapping.pdoVariable[rPdoMapping.pdoVariablesCount-1].npdos] = pdoNr;
      rPdoMapping.pdoVariable[rPdoMapping.pdoVariablesCount-1].npdos++;
    }
}

int PDOclient::handle( int pdoNr,  CanMessage& m )
{
  int i, idx;
  unsigned char * data = m.getData();

  for( i = 0; i < m.getDataLen(); i++ )  // here instead of 8: m.getDataLen()
    {            
      if(tPdoMapping.mask[pdoNr][i].maskPattern != 0 ) 
        {
          idx = tPdoMapping.mask[pdoNr][i].mappedVariableIndex;
          memcpy( tPdoMapping.pdoVariable[idx].dataPtr, 
                  data + tPdoMapping.mask[pdoNr][i].shift, tPdoMapping.mask[pdoNr][i].nBytes );  
          tPdoMapping.pdoVariable[idx].updateTime = HrTimer::hrtimer();          
        }
    }  
  return 0;
}

void PDOclient::sendMappedVariable( int mappedVariableIndex, void * value )
{
  setMappedVariableValue(mappedVariableIndex, value);

  // trigger all PDO, which are mapped the variable
  for( int i = 0; i < rPdoMapping.pdoVariable[mappedVariableIndex].npdos; i++ ) send(rPdoMapping.pdoVariable[mappedVariableIndex].pdos[i]);
}

void PDOclient::sendMappedVariable( MappedVariable *mv, void * value )
{
  setMappedVariableValue(mv, value);
  for( int i = 0; i < mv->npdos; i++ ) send( mv->pdos[i] );  // trigger all PDO, which are mapping the variable
}

void PDOclient::setMappedVariableValue( int mappedVariableIndex, void * value )
{
  rPdoMapping.pdoVariable[mappedVariableIndex].updateTime = HrTimer::hrtimer();  
  memcpy( rPdoMapping.pdoVariable[mappedVariableIndex].dataPtr, value, rPdoMapping.pdoVariable[mappedVariableIndex].dataSize ); 
}

void PDOclient::setMappedVariableValue( MappedVariable *mv, void * value )
{
  mv->updateTime = HrTimer::hrtimer(); 
  memcpy( mv->dataPtr, value, mv->dataSize );
}

void PDOclient::send( int pdoNr )
{
  CanMessage msg;

  build( pdoNr, &msg );
  cmc->submit( msg ); 
}

void PDOclient::build( int pdoNr, CanMessage * msg )
{
  unsigned char * bytes = msg->getData();
  long v;
  int dataLen = 0;

  for( int i = 0; i < 8; i++ )
    {
      if( rPdoMapping.mask[pdoNr][i].maskPattern != 0 )
        {
          v = rPdoMapping.mask[pdoNr][i].maskPattern & *((long*)(rPdoMapping.pdoVariable[rPdoMapping.mask[pdoNr][i].mappedVariableIndex].dataPtr));
          memcpy( bytes + rPdoMapping.mask[pdoNr][i].shift, (char*)&v, rPdoMapping.mask[pdoNr][i].nBytes );
          dataLen +=  rPdoMapping.mask[pdoNr][i].nBytes;
        }
    }
  msg->setDataLen( dataLen );
  msg->setId( rPdoId[pdoNr] ) ;
}


MappedVariable * PDOclient::getMappedVariable( PdoMapping * pdoMapping, int index, int subindex )
{
  for( int i = 0; i < pdoMapping->pdoVariablesCount; i++ )
    if(  pdoMapping->pdoVariable[i].index == index && pdoMapping->pdoVariable[i].subindex == subindex ) return &(pdoMapping->pdoVariable[i]);
  return NULL;
}
