
#ifndef __PDO_H__
#define __PDO_H__

#include "loggerClass/log.h"
#include <string.h>
extern Log Lg;

const int MAX_PDO = 16;   // max 16 Transmit and 16 Receive PDOs pro node
class MappedVariable
{
 public:
  int index;
  int subindex;
  void *dataPtr;
  long long updateTime;
  int dataSize;
  int npdos;          //number of PDOs, to which variable is mapped
  int pdos[MAX_PDO];  //PDOs which map this variable, used only for RPDO
  inline  MappedVariable() { updateTime = 0; npdos = 0; dataPtr = 0; memset( pdos, 0, sizeof(int)*MAX_PDO ); }
  inline  void write(int lval) { memcpy(dataPtr,&lval,dataSize);  } // 13.06.07 swh 
  inline void dump() { Lg << DBG <<"index = 0x" << hex << index <<", subindex = 0x" << subindex 
                            << ", data size = 0x" << dataSize << ", npdos = " << npdos 
                            << ", data Ptr = " << dataPtr << dec << " updated at: " << updateTime << endl; }
};


#include "canDataTypes.h"
#include "timeSlice.h"
#include "hrtimer.h"
#include "canMessageCollector.h"

class CanNodeImage;

typedef struct
{
  int shift;
  int maskPattern;
  int nBytes;
  int mappedVariableIndex;
} PdoMask;

typedef struct
{
  PdoMask mask[MAX_PDO][8];
  MappedVariable pdoVariable[MAX_PDO * 8];
  int pdoVariablesCount;
} PdoMapping;

class PDOclient : public TimeSlice
{
 private: 
  CanNodeImage * owner;  
  CanMessageCollector * cmc;
  int nodeID;
  int rPdoId[MAX_PDO];
  int tPdoId[MAX_PDO];
  PdoMapping tPdoMapping;  // transmit PDO (PDO sent by the remote node)
  PdoMapping rPdoMapping;  // receive PDO (PDO received by the remote node)
  int add( PdoMapping * pdoMapping, int pdoNr, int itemNr, int mapping );  
  void build( int pdoNr, CanMessage * msg ); 
  MappedVariable *getMappedVariable( PdoMapping * pdoMapping, int index, int subindex );
 public:
  PDOclient( CanNodeImage * owner);
  inline int setRpdoCobId( int pdoNr, int cobid ) { if( pdoNr < MAX_PDO ) rPdoId[pdoNr] = cobid; 
    Lg << DBG << "Setting COBID = 0x" << hex <<  cobid << dec << " for ReadPDO, pdoNr =  " << pdoNr << endl;  return (pdoNr < MAX_PDO ? 0: 1); } // overwrite predefined connection set COBID for RPDO
  inline int setTpdoCobId( int pdoNr, int cobid ) { if( pdoNr < MAX_PDO ) tPdoId[pdoNr] = cobid; 
    Lg << DBG << "Setting COBID = 0x" << hex <<  cobid << dec << " for TransmitPDO, pdoNr =  " << pdoNr << endl;  return (pdoNr < MAX_PDO ? 0: 1); } // overwrite predefined connection set COBID for TPDO  
  inline void reset() 
    { 
      memset( &tPdoMapping, '\0', sizeof(PdoMapping) ); 
      memset( &rPdoMapping, '\0', sizeof(PdoMapping) );
    }
  inline void connectMessageCollector( CanMessageCollector * collector ) { cmc = collector; }
  int handle( int pdoNr, CanMessage& m ); 
  inline void setNodeId( int id ) { nodeID = id; /* for( int i = 0; i < MAX_PDO; i++ ) rPdoId[i] = 0x200 + i * 0x100 + nodeID; */ }
  inline void addTpdoMapping( int pdoNr, int itemNr, int mapping ) {  add( &tPdoMapping, pdoNr, itemNr, mapping ); }
  void addRpdoMapping( int pdoNr, int itemNr, int mapping ); 
  inline int getTpdoVariablesCount() { return tPdoMapping.pdoVariablesCount; }  
  inline int getRpdoVariablesCount() { return rPdoMapping.pdoVariablesCount; }
  inline MappedVariable * getTmappedVariable( int varNo ) { return &(tPdoMapping.pdoVariable[varNo]); } 
  inline MappedVariable * getRmappedVariable( int varNo ) { return &(rPdoMapping.pdoVariable[varNo]); }
  inline MappedVariable *getTmappedVariable( int index, int subindex ) { return getMappedVariable( &tPdoMapping, index, subindex ); }
  inline MappedVariable *getRmappedVariable( int index, int subindex ) { return getMappedVariable( &rPdoMapping, index, subindex ); }
  void setMappedVariableValue(int mappedVariableIndex, void * value );
  void setMappedVariableValue(MappedVariable *mv, void * value );  
  void sendMappedVariable(int mappedVariableIndex, void * value );
  void sendMappedVariable(MappedVariable *mv, void * value );
  void send( int pdoNr );
  
  inline int getTpdoCobId(int pdoNr ) { return pdoNr < MAX_PDO ? tPdoId[pdoNr] : -1; } 
  inline int getRpdoCobId(int pdoNr ) { return pdoNr < MAX_PDO ? rPdoId[pdoNr] : -1; }
};

#endif
