

#ifndef __SDO_H__
#define __SDO_H__

#include "canDataTypes.h"
#include "timeSlice.h"
#include "hrtimer.h"
#include "canMessageCollector.h"

class SDOmessage
{
 public:
  int index;
  int subindex;
  char data[4];
  unsigned char commandSpecifier;
};

const int NO_ERROR = 0x00;
const int ERROR_TIMEOUT  = 0x01;   
const int ERROR_BAD_SDO_SEQUENCE = 0x02;
const int ERROR_BAD_TOGGLE_BIT = 0x03;
const int ERROR_ABORT = 0x04;
const int ERROR_SDO_BUSY = 0x05;
const int ERROR_UNHANDLED_SDO_TYPE = 0x06;
const int ERROR_NOT_ENOUGH_MEMORY = 0x07;

const int READ = 1;
const int WRITE = 2;

const int SDO_TIMEOUT_MILISEC = 5000;

const unsigned char INITIATE_UPLOAD_CODE = 0x40;
const unsigned char SEGMENT_UPLOAD_REQUEST_CODE = 0x60;
const unsigned char SEGMENT_UPLOAD_RESPONSE_CODE = 0x00;
const unsigned char INITIATE_DOWNLOAD_CODE = 0x20;
const unsigned char INITIATE_DOWNLOAD_RESPONSE_CODE = 0x60;
const unsigned char SEGMENT_DOWNLOAD_REQUEST_CODE = 0x00;
const unsigned char SEGMENT_DOWNLOAD_RESPONSE_CODE = 0x20;


const unsigned char ABORT_CODE = 0x80; 
 
class SDOclient : public TimeSlice
{
 private:
  enum 
    { IDLE = 0, 
      INITIATE_DOWNLOAD,
      DOWNLOAD_SEGMENT,
      INITIATE_UPLOAD,
      UPLOAD_SEGMENT,
      ABORT_TRANSFER,
      DONE
    };
  int index;
  int subindex;
  int nodeID;
  int receiveIdOffset ;
  int transmitIdOffset;
  int state;
  unsigned int error;
  int dataType;
  int access;
  char * dataStorage;
  char * localStorage;
  int  dataSize;
  int progress;
  int abortCode;
  int dataStorageOffset;
  unsigned int toggleBit;
  HrTimer * Timer;
  CanMessageCollector * cmc;

  void construct( int cobID, CanMessageCollector * collector );
  void convert();
  void requestNextSegment();
  int  isAbort( CanMessage& m );
  void initiateUpload( CanMessage& m );
  void uploadSegment( CanMessage& m );
  void initiateDownload( CanMessage& m );
  void downloadSegment( CanMessage& m ); 
  int sendDataSegment();
 public:
 
  SDOclient( int cobID = 1 );
  SDOclient( int cobID, CanMessageCollector * collector );
  int read( int idx, int subidx,  int dataSize ); 
  int read( int idx, int subidx, void * data, int dataType ); 
  int write( int idx, int subidx, void * data, int dataType );
  void activate();
  inline void connectMessageCollector( CanMessageCollector * collector ) { cmc = collector; }
  inline void setNodeId( int id ) { nodeID = id; }
  inline int getNodeId() { return nodeID; }
  inline void setTransmitIDoffset( int offs ) { transmitIdOffset = offs; }
  inline void setReceiveIDoffset( int offs ) { receiveIdOffset = offs; }
  inline int getTransmitIDoffset() { return transmitIdOffset; }
  inline int getReceiveIDoffset() { return receiveIdOffset; }
  int handle( CanMessage& m );
  inline int getState() { return state; }
  inline int done() { return state == DONE; }
  inline int available() { return state == IDLE; }
  inline void reset() { if( localStorage ) { free( localStorage ); localStorage = NULL; } state = IDLE; error = abortCode = 0; }
  inline int getError() { return error; }
  inline int getIndex() { return index; }
  inline int getSubindex() { return subindex; }
  inline int getDataType() { return dataType; }
  inline int getAccess() { return access; }
  inline char *getBuffer() { return dataStorage; } 
  inline int getDataSize() { return dataSize; }
  inline void setDataSize( int dSize ) { dataSize = dSize; }
  int  setDataStorage( char * data, int size = 512 );
  inline unsigned int getProgress() { return progress; }
 char * getErrorDescription();
 char * getErrorDescription( unsigned int errCode );
 inline int getAbortCode()  { return abortCode; }
 char * getAbortDescription();
 char * getAbortDescription(int errCode);
};

#endif
