
#include "sdoForwarder.h"

SDOforwarder::SDOforwarder( SDOclient * sdo )
{
    blocked = 0;
    sdoClient = sdo;
    state = IDLE;
    inputQueue.flush();
}

void SDOforwarder::activate()
{    
    switch( state )
    {
        case IDLE:           
            if( sdoClient->available() )
            {
                if( isBlocked() ) return;
                if( inputQueue.get( sdoFwd ) ) 
                {
                    Lg << DBG << "SDOforwarder: idle state, submitting: ";
                    sdoFwd.dump();
                    submitSDO( sdoFwd );
                }
            }
            break;              
        case BUSY:
            if( sdoClient->done() ) 
            {	
                Lg << DBG << "SDOforwarder: busy state, SDO done, updating result: " << sdoFwd.getResult() << " with 0x" << hex << (sdoClient->getError() | SDO_DONE) << " for index 0x" << sdoFwd.getIndex() << ", subindex = 0x" << sdoFwd.getSubindex() << dec << endl;
               *(sdoFwd.getResult()) = sdoClient->getError() | SDO_DONE;               
               sdoClient->reset();
               if( isBlocked() )
               {
                   removeAwaitingSDOs();
                   return;
               }
               if( inputQueue.get( sdoFwd ) ) 
               {
                    Lg << DBG << "SDOforwarder: busy state, submitting: ";
                    sdoFwd.dump();
                    submitSDO( sdoFwd );
               }
               else 
               {
                   Lg << DBG << "SDOforwarder: input queue is empty, setting IDLE state" << endl;
                   state = IDLE;
               }
            }
            break;            
    }    
}

void SDOforwarder::removeAwaitingSDOs()
{
   while( inputQueue.get(sdoFwd) ) *(sdoFwd.getResult()) = SDO_REJECTED | SDO_DONE;   
}

int SDOforwarder::submitSDO( SDOfwd& s )
{
   int errCode;
    if( s.getCommand() == WRITE )  
      {
        errCode = sdoClient->write( s.getIndex(), s.getSubindex(), s.getData(), s.getDataType() );
         Lg << DBG << "submitting SDO write, errCode = " << errCode << endl;
      }
   else
     {
       errCode = sdoClient->read( s.getIndex(), s.getSubindex(), s.getData(), s.getDataType() );
       Lg << DBG << "submitting SDO read, errCode = " << errCode << endl;
     }
   if( errCode == NO_ERROR ) state = BUSY;
   Lg << DBG << "state = " << state << endl;
   return errCode;
}


