#include "sequencer.h"
#include "canNodeImage.h"

Sequence::Sequence()
{
  data = buffer;
  dataSize = sizeof(buffer);
  dataCopy = bufferCopy;      
  dataCopyDynamic = 0;  
  nSteps = 0; 
}

Sequence::Sequence( void * dataStorage, int dataStorageSize )
{
  data = dataStorage;
  dataSize = dataStorageSize;
  if( data == NULL )
  {
      data = buffer;
      dataSize = sizeof(buffer);
      dataCopy = bufferCopy;      
      dataCopyDynamic = 0;
  }
  else
  {
      dataCopy = new char[dataSize];
      dataCopyDynamic = 1;
  }
  nSteps = 0;
}

int Sequence:: add(  int (CanNodeImage::*m)(int * i, void * data ) )
{
  if( nSteps == MAX_SEQUENCE_LEN ) 
    {
      Lg << FATAL << "---Fatal: can not add next sequence step, exiting..." << endl;
      exit(1);
    }
  step[nSteps++].set( m );
  return 1;
}


Sequencer::Sequencer( CanNodeImage * own, Sequence * s )
{
  sequence = NULL;
  owner = own;
  set(s);
}

void Sequencer::activate()
{
  if( state != SEQUENCER_RUNNING ) return;
  if( currentStep < sequence->getSize() )
    {
      prevCallsCounter = callsCounter;
      sequence->makeDataCopy();
      if( (error = sequence->execute( currentStep, &callsCounter)) == SEQUENCER_NO_ERROR ) 
        { 
          callsCounter = 0;
          if( ++currentStep == sequence->getSize()) state = SEQUENCER_DONE;
          return;
        }
      else 
      {
	  if( error == SEQUENCER_RETRY )      
	  {
	      sequence->restoreData();          // restore the data, since the last step must be repeated          
	      callsCounter = prevCallsCounter;  // restore the callsCounter, since the last step must be repeated          
	      return;
	  }
      }
      if( error == SEQUENCER_CONTINUE ) return;
      state = SEQUENCER_ERROR;
    }  
  else Lg << WARNING << "-Warning: Sequencer out of range" << endl;
  return;  
}

void Sequencer::set( Sequence * s ) 
{ 
  reset(); 
  sequence = s; 
  if( sequence ) sequence->setOwner( owner ); 
}

void Sequencer::reset() 
{ 
  if( sequence!=NULL ) delete sequence;
  currentStep = 0;  
  callsCounter = 0;
  error = 0; 
  state = SEQUENCER_IDLE; 
  sequence = NULL;  
}
