
#ifndef __SEQUENCER_H__
#define __SEQUENCER_H__ 

#include "loggerClass/log.h"
#include "timeSlice.h"
extern Log Lg;


class CanNodeImage;

class SequenceStep 
{
  private:
  int (CanNodeImage::*method)(int *i, void *data);
  CanNodeImage *object;
 
  public:
  inline void set(CanNodeImage * o, int (CanNodeImage::*m)(int *i, void * data ))
  {
    object = o;
    method = m;
  }
  inline void set(int (CanNodeImage::*m)(int *i, void * data ))
  {
    method = m;
  }
  inline void setOwner(CanNodeImage * owner)
  {
    object = owner;
  }
  inline int execute(int *i, void * data)
  {
    return (object->*method)(i, data);
  }
};

const int MAX_SEQUENCE_LEN = 50;
const int SEQUENCER_NO_ERROR = 0;
const int SEQUENCER_CONTINUE = 1;
const int SEQUENCER_ABORT = 2;
const int SEQUENCER_RETRY = 3;

const int SEQUENCER_IDLE = 0;
const int SEQUENCER_RUNNING = 1;
const int SEQUENCER_ERROR = 2;
const int SEQUENCER_DONE = 3;


class Sequence
{
 private:
  SequenceStep step[MAX_SEQUENCE_LEN];
  int    nSteps;
  void * data;
  void * dataCopy;
  int dataSize;
  int dataCopyDynamic;
  char buffer[256];  // some local storage for temporary data
  char bufferCopy[256];
  
 public:
  Sequence();   
  Sequence( void * dataStorage , int dataStorageSize );
  ~Sequence() { if(dataCopyDynamic) delete (char*)dataCopy; }
  inline void setOwner( CanNodeImage * own ) { for( int i = 0; i < nSteps; i++ ) step[i].setOwner(own); }
  int add(  int (CanNodeImage::*m)(int *i, void *data) );
  inline int getSize() { return nSteps; }
  inline void makeDataCopy() { memcpy( dataCopy, data, dataSize ); }
  inline void restoreData() { memcpy( data, dataCopy, dataSize); }
  inline int execute( int stepNo, int *callCounter ) { return step[stepNo].execute( callCounter, data ); }
};

class Sequencer : public TimeSlice
{
  private: 
  CanNodeImage * owner;
  Sequence * sequence;
  int currentStep;
  int prevCallsCounter;
  int callsCounter;
  int error;
  int state;
 
  public:  
  Sequencer( CanNodeImage * own,  Sequence * s = NULL );
  inline ~Sequencer() { reset(); };
  void set( Sequence * s );
  inline Sequence * getSequence() { return sequence; }
  void reset();
  inline int getError() { return error; }
  inline int getState() { return state; }
  inline bool done() { return ((state == SEQUENCER_DONE) || (state == SEQUENCER_ERROR)); }
  inline bool available() { return state == SEQUENCER_IDLE; }
  inline void run() { state = SEQUENCER_RUNNING; }
  void activate();
};


#endif
