
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>
#include <signal.h>


#include "shmlogger.h"


#define DEFAULT_PATH "/var"
#define MAX_FILES_PRO_DAY 3
#define MAX_FILE_SIZE 1000000L
#define MAX_LINE (60+MAX_MESSAGE + MAX_PROCESS_NAME)


static void  cleanup( void );
static void  terminate( int signo );
static void  displayMsg( LOGMSG *msh );
static int   crack( char * str, char * words[], char * delims, int maxwords );
static int   output_bold( char * line );


static char  Format[64];

int main( void )
{
  int errCode;
  long currentRead,  rdPtr, logSize;
  LOGMSG  msg;

 
  signal( SIGINT,  terminate );
  signal( SIGQUIT, terminate );
  signal( SIGABRT, terminate );
  signal( SIGTERM, terminate );
  signal( SIGTSTP, terminate );

  atexit( cleanup );
  
  printf( "logdisp vers. %s, %s started.\n", __DATE__, __TIME__ );

  if( (errCode = openLog("logdisp")) != 0 )
    {
      printf( "openLog() error = %d, %s\n", errCode, strerror(errCode));
      exit(errCode);
    }

  sprintf( Format, "%s%d%s", "%02d.%02d.%02d %02d:%02d:%02d %", MAX_PROCESS_NAME, "s [%d]: %s");

  currentRead = 0;
  while( getLogInfo( &currentRead, &logSize ) == 0 ) usleep(300000L);

  while( 1 )
    {

      rdPtr = currentRead;
      while( getLogInfo( &rdPtr, &logSize ) )  usleep(300000L);
      do
	{
	  if( ++currentRead == logSize ) currentRead = 0;
	   readLog( currentRead, &msg );
	   displayMsg( &msg );	   
	}while( currentRead != rdPtr );
    }  
}  


void terminate( int signo )
{
    exit(0);
}

void cleanup( void )
{
    closeLog();
}

void displayMsg( LOGMSG * msg )
{
  char buff[MAX_LINE+1];
  struct tm * t  = localtime( &( msg->timeStamp ) );

  sprintf( buff, Format , t->tm_mday, t->tm_mon+1, t->tm_year%100, t->tm_hour, t->tm_min, t->tm_sec,
	   msg->pName, msg->pid, msg->message);
  if( output_bold( msg->message ) )
    {
      printf( "%c[1m",27 );   /* bold ON */
      printf( "%s\n", buff );            
      printf( "%c[0m",27 );   /* all atributes off */                
    }
  else puts( buff );
}

#define BOLD_WORDS 3
int output_bold( char * line )
{
    int n;
    int i,j;
    char buff[MAX_MESSAGE + 1];
    char *words[10];
    static char * bw[BOLD_WORDS] = { "*E", "*W", "--"  };

    strcpy( buff, line  );
    n = crack( buff, words," \n\r", 10 );
    for( i = 0; i < BOLD_WORDS; i++ )
         for( j = 0; j < n; j++ )
            if( !strncmp( bw[i], words[j],2 ) ) return 1;
      
    return 0;
}

int crack( char * str, char * words[], char * delims, int maxwords )
{
    int n = 0;

    if( ( words[0] = strtok( str, delims ) ) != NULL )
       while( ++n < maxwords ) if( (words[n] = strtok( NULL, delims )) == NULL ) break;
     
    return n;
}
