

#include "sync.h"


SyncProducer::SyncProducer(CanMessageCollector * collector)
{
  cmc = collector;
  enabled = false;
  interval = 1000;
  syncId = 0x80;   //default

  setName( "SYNC" );
  setFamilyName( "SYNCPRODUCER" );
  
  DTYPE out, in;
  out.dArrayLength = 1;  out.dFormat = CF_LONG; 
  in.dArrayLength = 1;  in.dFormat = CF_LONG;
  addProperty("period",&out, &in, CA_READ | CA_WRITE, "SYNC message period (ms)",  &SyncProducer::property_Period);
  addProperty("on_off", &out, &in, CA_READ | CA_WRITE, "SYNC genaration on(1), off(0)",&SyncProducer::property_OnOff); 
  
  out.dArrayLength = 2;  out.dFormat = CF_LONG; 
  in.dArrayLength = 0;  in.dFormat = CF_NULL;
  addProperty("periodOnOff", &out, &in, CA_READ | CA_WRITE, "item[0]: onoff, [1]: period [ms]",&SyncProducer::property_periodOnOff);
}

void SyncProducer::addProperty( char * propName, DTYPE * out, DTYPE * in, short acc, char * dsc, int (SyncProducer::*h)( TINE_ARGS ))
{
  strcpy( propertyHandlerList[propertyCounter].name, propName);
  propertyHandlerList[propertyCounter].handler = h;
  TineDevice::addProperty(propName, out, in, acc, dsc, NULL );
} 

void SyncProducer::activate()
{
  if( !enabled ) return;

  if( Timer.expired() )
    {
      CanMessage m;     
      m.setDataLen(0);
      m.setId( syncId );
      cmc->submit(m);
      Timer.restart();
    }
}

int SyncProducer::propertyHandler( char *property, TINE_ARGS )
{
  for( int i = 0; i < propertyCounter; i++ )        
    {
      if( !strcmp( property, propertyHandlerList[i].name) ) return ( this->*propertyHandlerList[i].handler)( dout, din, access ); 
    }

  return 0;
}

int SyncProducer::property_Period( TINE_ARGS )
{
  if( access & CA_READ )
    {
      if( dout == NULL ) return illegal_data_size ;
      if( dout->dFormat != CF_LONG  ) return illegal_format;
      if( dout->dArrayLength < 1 ) return illegal_data_size;
      *(dout->data.lptr) = getInterval();
    }
  if( access & CA_WRITE )
    {    
      if( din == NULL ) return illegal_data_size ;
      if( din->dFormat != CF_LONG  ) return illegal_format;
      if( din->dArrayLength < 1 ) return illegal_data_size;     
      if( *(din->data.lptr) < 1 ) return invalid_data;      
      setInterval(*(din->data.lptr)); 
    }
  return 0;;
}


int SyncProducer::property_OnOff(  TINE_ARGS )
{ 
  int oldEnabled = isEnabled();

  if( access & CA_WRITE )
    {     
      Lg << DBG << "SYNC property_onOff called, access write." << endl;
      if( din == NULL ) return illegal_data_size ;
      Lg << DBG << "data format = " << din->dFormat << ", " << ", size = " << din->dArrayLength << endl;
      if( din->dFormat != CF_LONG  ) return illegal_format;
      if( din->dArrayLength < 1 ) return illegal_data_size;     
      if(*(din->data.lptr)) enable(); 
      else disable(); 
    }
 if( access & CA_READ )
    {
      if( dout == NULL ) return illegal_data_size ;
      if( dout->dFormat != CF_LONG  ) return illegal_format;
      if( dout->dArrayLength < 1 ) return illegal_data_size;
      *(dout->data.lptr) = oldEnabled;
    }
  return 0;
}

int SyncProducer::property_periodOnOff( TINE_ARGS )
{ 
  int oldInterval = getInterval();
  int oldEnabled = isEnabled();

  if( access & CA_WRITE )
    {     
      if( din == NULL ) return illegal_data_size ;
      if( din->dFormat != CF_LONG  ) return illegal_format;
      if( din->dArrayLength < 2 ) return illegal_data_size;     
      if((din->data.lptr)[0]) enable();   else disable();       
      setInterval((din->data.lptr)[1]);
    }
  if( access & CA_READ )
    {
      if( dout == NULL ) return illegal_data_size ;
      if( dout->dFormat != CF_LONG  ) return illegal_format;
      if( dout->dArrayLength < 2 ) return illegal_data_size;
      (dout->data.lptr)[0] = oldEnabled;
      (dout->data.lptr)[1] = oldInterval;
      }
  return 0;
}

void SyncProducer::dump()
{
  Lg << DBG << endl << "Dump of SyncProducer:" << endl;
  Lg << DBG << "On/Off: " << isEnabled() << endl;
  Lg << DBG << "Period: " << getInterval() << endl;
  Lg << DBG << "COB ID: 0x" << hex << getId() << dec << endl; 
  Lg << DBG << endl;
}
