

#include "timeProducer.h"


TimeProducer::TimeProducer(CanMessageCollector * collector)
{
  struct tm t;

  memset( &t, '\0', sizeof(struct tm) );
  t.tm_year = 84;
  t.tm_mon = 0;
  t.tm_mday = 1;
  ReferenceDate = mktime( &t );

  cmc = collector;
  enabled = false;
  interval = 1000;
  timeProducerId = 0x100;   //default

  setName( "TIMEPRODUCER" );
  setFamilyName( "TIMEPRODUCER" );
   
  DTYPE out, in;
  out.dArrayLength = 1;  out.dFormat = CF_LONG; 
  in.dArrayLength = 1;  in.dFormat = CF_LONG;
  
  addProperty("period",&out, &in, CA_READ | CA_WRITE, "TIMEPRODUCER message period (ms)",  &TimeProducer::property_Period);
  addProperty("on_off", &out, &in, CA_READ | CA_WRITE, "TIME stamp genaration on(1), off(0)",&TimeProducer::property_OnOff); 
  
  out.dArrayLength = 2;  out.dFormat = CF_LONG; 
  in.dArrayLength = 0;  in.dFormat = CF_NULL;
  
  addProperty("periodOnOff", &out, &in, CA_READ | CA_WRITE, "item(0): onoff, (1): period (ms)",&TimeProducer::property_periodOnOff); 
  addProperty("timeStamp", &out, &in, CA_READ, "item(0): timeStamp (ms) , (1):  dateStamp (days)",&TimeProducer::property_timeStamp);
}

void TimeProducer::addProperty( char * propName, DTYPE * out, DTYPE * in, short acc, char * dsc, int (TimeProducer::*h)( TINE_ARGS ))
{
  strcpy( propertyHandlerList[propertyCounter].name, propName);
  propertyHandlerList[propertyCounter].handler = h;
  TineDevice::addProperty(propName, out, in, acc, dsc, NULL );
} 

void TimeProducer::activate()
{
  if( !enabled ) return;

  if( Timer.expired() )
    {
      
      CanMessage m;     
      struct timeval tb;
      long long int tsec;
      gettimeofday( &tb, NULL );
      tsec = tb.tv_sec;
      struct tm * tms = localtime((time_t*) &tsec);  
      recentTimeStamp = ((tms->tm_hour * 60 + tms->tm_min) * 60 + tms->tm_sec ) * 1000 +  tb.tv_usec/1000;
      recentDateStamp = (long)(difftime(tsec, ReferenceDate) / (24 * 3600));  //in days

      unsigned char * data = m.getData();
      m.setDataLen(6);
      m.setId( timeProducerId );
      data[0] = recentTimeStamp & 0xFF;
      data[1] = (recentTimeStamp >> 8)  & 0xFF;
      data[2] = (recentTimeStamp >> 16) & 0xFF;
      data[3] = (recentTimeStamp >> 24) & 0x0F;  // 28 bits for miliseconds since midnight
      data[4] = recentDateStamp & 0xFF;
      data[5] = (recentDateStamp >> 8 ) & 0xFF;
      cmc->submit(m);
      Timer.restart();
    }
}

int TimeProducer::propertyHandler( char *property, TINE_ARGS )
{
  for( int i = 0; i < propertyCounter; i++ )        
    {
      if( !strcmp( property, propertyHandlerList[i].name) ) return ( this->*propertyHandlerList[i].handler)( dout, din, access ); 
    }

  return 0;
}

int TimeProducer::property_Period( TINE_ARGS )
{
  if( access & CA_READ )
    {
      if( dout == NULL ) return illegal_data_size ;
      if( dout->dFormat != CF_LONG  ) return illegal_format;
      if( dout->dArrayLength < 1 ) return illegal_data_size;
      *(dout->data.lptr) = getInterval();
    }
  if( access & CA_WRITE )
    {    
      if( din == NULL ) return illegal_data_size ;
      if( din->dFormat != CF_LONG  ) return illegal_format;
      if( din->dArrayLength < 1 ) return illegal_data_size;     
      if( *(din->data.lptr) < 1 ) return invalid_data;      
      setInterval(*(din->data.lptr)); 
    }
  return 0;;
}


int TimeProducer::property_OnOff( TINE_ARGS )
{ 
  if( access & CA_READ )
    {
      if( dout == NULL ) return illegal_data_size ;
      if( dout->dFormat != CF_LONG  ) return illegal_format;
      if( dout->dArrayLength < 1 ) return illegal_data_size;
      *(dout->data.lptr) = ( isEnabled() ? 1 : 0 );
      }
  if( access & CA_WRITE )
    {     
      if( din == NULL ) return illegal_data_size ;
      if( din->dFormat != CF_LONG  ) return illegal_format;
      if( din->dArrayLength < 1 ) return illegal_data_size;     
      if(*(din->data.lptr)) enable(); 
      else disable(); 
    }

  return 0;
}

int TimeProducer::property_periodOnOff( TINE_ARGS )
{ 
  if( access & CA_READ )
    {
      if( dout == NULL ) return illegal_data_size ;
      if( dout->dFormat != CF_LONG  ) return illegal_format;
      if( dout->dArrayLength < 2 ) return illegal_data_size;
      (dout->data.lptr)[0] = ( isEnabled() ? 1 : 0 );
      (dout->data.lptr)[1] = getInterval();
      }
  if( access & CA_WRITE )
    {     
      if( din == NULL ) return illegal_data_size ;
      if( din->dFormat != CF_LONG  ) return illegal_format;
      if( din->dArrayLength < 2 ) return illegal_data_size;     
      if((din->data.lptr)[0]) enable();   else disable();       
      setInterval((din->data.lptr)[1]);
    }

  return 0;
}

int TimeProducer::property_timeStamp( TINE_ARGS )
{ 
  if( access & CA_WRITE ) return illegal_read_write;
  if( access & CA_READ )
    {
      if( dout == NULL ) return illegal_data_size ;
      if( dout->dFormat != CF_LONG  ) return illegal_format;
      if( dout->dArrayLength < 2 ) return illegal_data_size;
      (dout->data.lptr)[0] = recentTimeStamp;
      (dout->data.lptr)[1] = recentDateStamp;
      }
  return 0;
}

void TimeProducer::dump()
{
  Lg << DBG << endl << "Dump of TimeProducer:" << endl;
  Lg << DBG << "On/Off: " << isEnabled() << endl;
  Lg << DBG << "Period: " << getInterval() << endl;
  Lg << DBG << "COB ID: 0x" << hex << getId() << dec << endl; 
  Lg << DBG << endl;
}
