
#ifndef __TINESERVER_H__
#define __TINESERVER_H__


extern "C"
{
#include <stdio.h>   // for "FILE*" - ask PD.
#include "tine.h"
}

#include "timeSlice.h"
#include "configurator.h"


#define TINE_ARGS DTYPE* dout, DTYPE* din, short access

const int MAX_PROPERTY = 100;
const int MAX_TINE_DEVICES = 140;  /* up to 12 various objects besides the Node Images */
const int MAX_TINE_EQUIPMENT_MODULES  = 10;

class TineDevice;
class TineServer;

class TineHandler
{
    public:
    virtual int getBackgroundRate() = 0;
    virtual int eqmHandler(char *deviceName, char *property, TINE_ARGS) = 0;
    virtual int eqmInit() = 0;
    virtual int eqmExit() = 0;
    virtual int eqmBackground() = 0;
    
};

class DefaultTineHandler:public TineHandler
{
    private:
    TineServer * srv;  
    int backgroundRate;
    
    public:
    inline DefaultTineHandler() {backgroundRate = 100;  srv = NULL;}
    inline DefaultTineHandler( TineServer * t ) { backgroundRate = 100; srv = t; }
    inline void setServer( TineServer * t ) { srv = t; }
    inline void setBackgroundRate( int br ) { backgroundRate = br; }
    inline int  getBackgroundRate() { return backgroundRate; }
    virtual int eqmHandler(char *deviceName, char *property, TINE_ARGS);    
    inline int eqmInit() { return 0; }
    inline int eqmExit() { return 0; }
    inline int eqmBackground() { return 0; }
};


class TineProperty
{
private:
  char  name[33];
  int access;
  char  description[65];
  DTYPE dout;
  DTYPE din;
  short aType;
  short rowlength;
  char  redirection[128];
  int   propertyId;
  void * userData;
public:
  TineProperty() { }
  TineProperty(const char* pname, DTYPE * out, DTYPE * in, int acc, const char* desc, void* udata) 
  {   
    setName(pname);
    setAccess(acc); 
    setDataOutInfo(out);
    setDataInInfo(in);
    setDescription(desc); 
    setUserData(udata); 
    setAtype(0);
    setRowLength(0);
    setRedirection("");
    setPropertyId(0);
  }
  
  TineProperty(const char* pname,DTYPE * out, DTYPE * in,int acc, const char* desc) 
  {
   setName(pname);
    setAccess(acc); 
    setDataOutInfo(out);
    setDataInInfo(in);
    setDescription(desc); 
    setUserData(0); 
    setAtype(0);
    setRowLength(0);
    setRedirection("");
    setPropertyId(0);
  }
  
  inline void setAccess(int acc) { access = acc; }
  inline void setDataInInfo( DTYPE * in) {  memcpy( &din, in, sizeof(DTYPE)); };
  inline void setDataOutInfo( DTYPE * out) {  memcpy( &dout, out, sizeof(DTYPE)); };
  inline void setName( const char * pname ) { strncpy(name,pname,32); name[32] = '\0'; }
  inline void setDescription(const char * desc ) { strncpy(description,desc,64); description[64] ='\0'; }
  inline void setUserData( void * udata ) { userData = udata; }
  inline void setAtype( short at ) { aType = at; }
  inline void setRowLength( short rl ) { rowlength = rl; }
  inline void setRedirection( char * redirect ) { strncpy(redirection, redirect, 127);  redirection[127] = '\0'; }
  inline void setPropertyId( int pid ) {  propertyId = pid; }
  
  inline char* getName() { return name; } 
  inline int   getAccess() { return access; }
  inline DTYPE* getDataInInfo() {  return &din; }
  inline DTYPE* getDataOutInfo() {  return &dout; }
  inline char* getDescription() { return description; }
  inline void* getUserData() { return userData; }
  inline short getAtype() { return aType; }
  inline short getRowLength() { return rowlength; }
  inline char* getRedirection() { return redirection; }
  inline int   getPropertyId() {  return propertyId; }
};


class TineDevice : public TimeSlice
{
 private:
  char name[33];
  char familyName[33];

 protected:
  int propertyCounter;
  TineProperty properties[MAX_PROPERTY]; 
  TineServer *  server;
 public:
  TineDevice();
  TineDevice( char * deviceName );  
  inline void setServer( TineServer * srv ) { server = srv; }
  inline void setName( char * n ) { strncpy( name, n, 32 );  name[32] = '\0'; }
  inline char * getName() { return name; }
  inline void setFamilyName( char * n ) { strncpy( familyName, n, 32 );  name[32] = '\0'; }
  inline char * getFamilyName() { return familyName; }
  inline TineProperty * getProperty( int i ) { return &properties[i]; }
  TineProperty * getProperty( char *propertyName );
  inline int getPropertyCount() { return propertyCounter; }
  inline void removeProperties() { propertyCounter = 0; }
  virtual void addProperty( char * propName, DTYPE * out, DTYPE * in, short acc, char * dsc, void * data );
  virtual int propertyHandler( char *property, TINE_ARGS ) = 0;
  void setEqmCompletion( char * message ); 
};

class TineFec
{
 private:  
  static unsigned short portOffset;
  static char fecName[17];
  static char context[17];
  static char description[33];
  static char operatingSystem[33];
  static char location[33];
  static char hardware[33];
  static char responsible[33];
 
 public:
  static int setDefaults();               // defaults for Polling rate etc. AND Fecname etc.
  static void configure( Configurator * config );
  static int init() { return init(0); }   // default to set values
  static int init(int mode);              // 0= use set values, 1= use fecid.csv

  static inline void setFecName( char * fn ) { strncpy( fecName, fn, 16 );  fecName[16] = '\0'; }
  static inline void setOperatingSystem(char * os ) { strncpy( operatingSystem, os, 32 ); operatingSystem[32] = '\0'; }
  static inline void setHardware( char * h ) { strncpy( hardware, h, 32 );  hardware[32] = '\0'; }  
  static inline void setLocation( char * l ) { strncpy( location, l, 32 );  location[32] = '\0'; }
  static inline void setResponsible( char * r ) { strncpy( responsible, r, 32 );  responsible[32] = '\0'; }
  static inline void setPortOffset( int po ) { portOffset = po; }
  static inline char * getFecName() { return fecName; }
  static inline char * getOperatingSystem() { return operatingSystem; }
  static inline char * getHardware() { return hardware; }
  static inline char * getLocation() { return location; }
  static inline char * getResponsible() { return responsible; }
  static inline int    getPortOffset() { return portOffset; }
  static inline void setDescription( char * d ) { strncpy( description, d, 32 );  description[32] = '\0'; } 
  static inline char * getDescription() { return description; } 
  static inline void setContext( char * c ) { strncpy( context , c, 16 );  context[16] = '\0'; } 
  static inline char * getContext() { return context; }
  static inline void setRemoteExit(int allowed) { if (allowed) gAllowRemoteExit = TRUE; }
};

class TineServer
{
 private:

  char serverName[32];
  char eqmTag[7];
  int  eqmBackgroundRate;
  int  eqmIndex;
  int  (* eqmHandler)    (char *, char *, TINE_ARGS );
  void (* eqmInit)       (void);
  void (* eqmBackground) (void); 
  void (* eqmExit)       (void);
  
  DefaultTineHandler defaultTineHandler;

protected:
  int deviceCounter;
  TineDevice * device[MAX_TINE_DEVICES]; 
  int propertyCounter;
 
 public:
  TineServer();
  TineServer( char * serverName );
  TineServer( char * serverName, char * eqpName) ;
  TineServer( char * serverName, char * eqpName, int mode); // test swh
  void construct( char * serverName );
  void construct( char * serverName, char *eqpName );
  void setServerName( char * sn );
  void setServerName( char * sn, char * eqpName );
  void setEqmName(char* eqpTag); 

  inline char * getServerName() { return serverName; }
  inline char * getEqmName() { return eqmTag; }
  
  inline int addProperty( TineProperty * p)
  {
    return  RegisterPropertyInformation( getEqmName(), 
                                        p->getName(), 
                                        p->getDataOutInfo(), 
                                        p->getDataInInfo(), 
                                        p->getAccess(), 
                                        p->getAtype(),
                                        p->getRowLength(),
                                        p->getDescription(),
                                        ++propertyCounter,
                                        p->getRedirection() );
  }
  //inline int addProperty( char * propName, int size, short fmt, short acc, char * dsc )
  //  {
  //    return RegisterPropertyEx( getEqmName(),  propName,  size,  fmt,  acc,  dsc, ++propertyCounter );
  //  }

  virtual int addDevice( TineDevice * device );
  int assignPropertyList(TineDevice * dev);
  int init(void (* eqmInit)(void), int (* eqmHandler)(char *, char *, DTYPE *, DTYPE *, short), // SWH 11.01.08
           void (* eqmBackground)(void),void (* eqmExit)(void), int bgrRate );
  inline int init() { defaultTineHandler.setServer(this); return init( &defaultTineHandler ); }
  int init( TineHandler* handler );
  inline TineDevice ** getDevices() { return device; }
  inline int getDeviceCount() { return deviceCounter; }

  inline void setEqmHandler( int  (* handler)(char *, char *, TINE_ARGS )) { eqmHandler = handler; }
  inline void setEqmInit( void  (* handler)()) { eqmInit = handler; }
  inline void setEqmBackground(  void  (* handler)()) { eqmBackground = handler; }
  inline void setEqmExit( void  (* handler)()) { eqmExit = handler; }
  inline void setEqmBackgroundRate( int rate ) { eqmBackgroundRate = rate; }
  inline int getEqmBackgroundRate() { return eqmBackgroundRate; } // SWH
  
  // for C callbaks
  
  static int nEquipmentModules;
  static TineHandler * tineHandler[MAX_TINE_EQUIPMENT_MODULES];
  
  static int (*eqmHandlerWrapper[MAX_TINE_EQUIPMENT_MODULES])( char *deviceName, char *property, TINE_ARGS); 
  static inline int eqmHandlerWrapper0( char *deviceName, char *property, TINE_ARGS);
  static inline int eqmHandlerWrapper1( char *deviceName, char *property, TINE_ARGS);
  static inline int eqmHandlerWrapper2( char *deviceName, char *property, TINE_ARGS);
  static inline int eqmHandlerWrapper3( char *deviceName, char *property, TINE_ARGS);
  static inline int eqmHandlerWrapper4( char *deviceName, char *property, TINE_ARGS);
  static inline int eqmHandlerWrapper5( char *deviceName, char *property, TINE_ARGS);
  static inline int eqmHandlerWrapper6( char *deviceName, char *property, TINE_ARGS);
  static inline int eqmHandlerWrapper7( char *deviceName, char *property, TINE_ARGS);
  static inline int eqmHandlerWrapper8( char *deviceName, char *property, TINE_ARGS);
  static inline int eqmHandlerWrapper9( char *deviceName, char *property, TINE_ARGS);
  
  static void (*eqmInitWrapper[MAX_TINE_EQUIPMENT_MODULES])();   
  static inline void eqmInitWrapper0();
  static inline void eqmInitWrapper1();
  static inline void eqmInitWrapper2();
  static inline void eqmInitWrapper3();
  static inline void eqmInitWrapper4();
  static inline void eqmInitWrapper5();
  static inline void eqmInitWrapper6();
  static inline void eqmInitWrapper7();
  static inline void eqmInitWrapper8();
  static inline void eqmInitWrapper9();
  
  static void (*eqmExitWrapper[MAX_TINE_EQUIPMENT_MODULES])(); 
  static inline void eqmExitWrapper0();
  static inline void eqmExitWrapper1();
  static inline void eqmExitWrapper2();
  static inline void eqmExitWrapper3();
  static inline void eqmExitWrapper4();
  static inline void eqmExitWrapper5();
  static inline void eqmExitWrapper6();
  static inline void eqmExitWrapper7();
  static inline void eqmExitWrapper8();
  static inline void eqmExitWrapper9();
  
  static void (*eqmBackgroundWrapper[MAX_TINE_EQUIPMENT_MODULES])(); 
  static inline void eqmBackgroundWrapper0();
  static inline void eqmBackgroundWrapper1();
  static inline void eqmBackgroundWrapper2();
  static inline void eqmBackgroundWrapper3();
  static inline void eqmBackgroundWrapper4();
  static inline void eqmBackgroundWrapper5();
  static inline void eqmBackgroundWrapper6();
  static inline void eqmBackgroundWrapper7();
  static inline void eqmBackgroundWrapper8();
  static inline void eqmBackgroundWrapper9();
  
};



#endif
